"""Signer interface and the default implementations"""

import logging
import os
from abc import ABCMeta, abstractmethod
from typing import Callable, Dict, Optional, Type
from urllib import parse

import securesystemslib.keys as sslib_keys
from securesystemslib.signer._key import Key, SSlibKey
from securesystemslib.signer._signature import Signature

logger = logging.getLogger(__name__)

# NOTE Signer dispatch table is defined here so it's usable by Signer,
# but is populated in __init__.py (and can be appended by users).
SIGNER_FOR_URI_SCHEME: Dict[str, Type] = {}


# SecretsHandler is a function the calling code can provide to Signer:
# SecretsHandler will be called if Signer needs additional secrets.
# The argument is the name of the secret ("PIN", "passphrase", etc).
# Return value is the secret string.
SecretsHandler = Callable[[str], str]


class Signer(metaclass=ABCMeta):
    """Signer interface that supports multiple signing implementations.

    Usage example:

        signer = Signer.from_priv_key_uri("envvar:MYPRIVKEY", pub_key)
        sig = signer.sign(b"data")

    Note that signer implementations may raise errors (during both
    Signer.from_priv_key_uri() and Signer.sign()) that are not documented here:
    examples could include network errors or file read errors. Applications
    should use generic try-except here if unexpected raises are not an option.

    See SIGNER_FOR_URI_SCHEME for supported private key URI schemes. The
    currently supported default schemes are:
    * envvar: see SSlibSigner for details
    * file: see SSlibSigner for details

    Interactive applications may also define a secrets handler that allows
    asking for user secrets if they are needed:

        from getpass import getpass

        def sec_handler(secret_name:str) -> str:
            return getpass(f"Enter {secret_name}: ")

        # user will not be asked for a passphrase for unencrypted key
        uri = "file:keys/mykey?encrypted=false"
        signer = Signer.from_priv_key_uri(uri, pub_key, sec_handler)

        # user will be asked for a passphrase for encrypted key
        uri2 = "file:keys/myenckey?encrypted=true"
        signer2 = Signer.from_priv_key_uri(uri2, pub_key2, sec_handler)

    Applications can provide their own Signer and Key implementations:

        from securesystemslib.signer import Signer, SIGNER_FOR_URI_SCHEME
        from mylib import MySigner

        SIGNER_FOR_URI_SCHEME[MySigner.MY_SCHEME] = MySigner

    This way the application code using signer API continues to work with
    default signers but now also uses the custom signer when the proper URI is
    used.
    """

    @abstractmethod
    def sign(self, payload: bytes) -> Signature:
        """Signs a given payload by the key assigned to the Signer instance.

        Arguments:
            payload: The bytes to be signed.

        Returns:
            Returns a "Signature" class instance.
        """
        raise NotImplementedError  # pragma: no cover

    @classmethod
    @abstractmethod
    def from_priv_key_uri(
        cls,
        priv_key_uri: str,
        public_key: Key,
        secrets_handler: Optional[SecretsHandler] = None,
    ) -> "Signer":
        """Factory constructor for a given private key URI

        Returns a specific Signer instance based on the private key URI and the
        supported uri schemes listed in SIGNER_FOR_URI_SCHEME.

        Args:
            priv_key_uri: URI that identifies the private key
            public_key: Key that is the public portion of this private key
            secrets_handler: Optional function that may be called if the
                signer needs additional secrets (like a PIN or passphrase).
                secrets_handler should return the requested secret string.

        Raises:
            ValueError: Incorrect arguments
            Other Signer-specific errors: These could include OSErrors for
                reading files or network errors for connecting to a KMS.
        """

        scheme, _, _ = priv_key_uri.partition(":")
        if scheme not in SIGNER_FOR_URI_SCHEME:
            raise ValueError(f"Unsupported private key scheme {scheme}")

        signer = SIGNER_FOR_URI_SCHEME[scheme]
        return signer.from_priv_key_uri(
            priv_key_uri, public_key, secrets_handler
        )


class SSlibSigner(Signer):
    """A securesystemslib signer implementation.

    Provides a sign method to generate a cryptographic signature with a
    securesystemslib-style rsa, ed25519 or ecdsa key. See keys module
    for the supported types, schemes and hash algorithms.

    SSlibSigners should be instantiated with Signer.from_priv_key_uri().
    These private key URI schemes are supported:
    * "envvar:<VAR>":
        VAR is an environment variable with unencrypted private key content.
           envvar:MYPRIVKEY
    * "file:<PATH>?encrypted=[true|false]":
        PATH is a file path to a file with private key content. If
        encrypted=true, the file is expected to have been created with
        securesystemslib.keys.encrypt_key().
           file:path/to/file?encrypted=true
           file:/abs/path/to/file?encrypted=false

    Attributes:
        key_dict:
            A securesystemslib-style key dictionary. This is an implementation
            detail, not part of public API
    """

    ENVVAR_URI_SCHEME = "envvar"
    FILE_URI_SCHEME = "file"

    def __init__(self, key_dict: Dict):
        self.key_dict = key_dict

    @classmethod
    def from_priv_key_uri(
        cls,
        priv_key_uri: str,
        public_key: Key,
        secrets_handler: Optional[SecretsHandler] = None,
    ) -> "SSlibSigner":
        """Constructor for Signer to call

        Please refer to Signer.from_priv_key_uri() documentation.

        Additionally raises:
            OSError: Reading the file failed with "file:" URI
        """
        if not isinstance(public_key, SSlibKey):
            raise ValueError(f"Expected SSlibKey for {priv_key_uri}")

        uri = parse.urlparse(priv_key_uri)

        if uri.scheme == cls.ENVVAR_URI_SCHEME:
            # read private key from environment variable
            private = os.getenv(uri.path)
            if private is None:
                raise ValueError(f"Unset env var for {priv_key_uri}")

        elif uri.scheme == cls.FILE_URI_SCHEME:
            params = dict(parse.parse_qsl(uri.query))
            if "encrypted" not in params:
                raise ValueError(f"{uri.scheme} requires 'encrypted' parameter")

            # read private key (may be encrypted or not) from file
            with open(uri.path, "rb") as f:
                private = f.read().decode()

            if params["encrypted"] != "false":
                if not secrets_handler:
                    raise ValueError("encrypted key requires a secrets handler")

                secret = secrets_handler("passphrase")
                decrypted = sslib_keys.decrypt_key(private, secret)
                private = decrypted["keyval"]["private"]

        else:
            raise ValueError(f"SSlibSigner does not support {priv_key_uri}")

        keydict = public_key.to_securesystemslib_key()
        keydict["keyval"]["private"] = private
        return cls(keydict)

    def sign(self, payload: bytes) -> Signature:
        """Signs a given payload by the key assigned to the SSlibSigner instance.

        Please see Signer.sign() documentation.

        Additionally raises:
            securesystemslib.exceptions.FormatError: Key argument is malformed.
            securesystemslib.exceptions.CryptoError, \
                securesystemslib.exceptions.UnsupportedAlgorithmError:
                Signing errors.
        """
        sig_dict = sslib_keys.create_signature(self.key_dict, payload)
        return Signature(**sig_dict)



"""
<Program>
  unittest_toolbox.py

<Author>
  Konstantin Andrianov.

<Started>
  March 26, 2012.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Provides an array of various methods for unit testing.  Use it instead of
  actual unittest module.  This module builds on unittest module.
  Specifically, Modified_TestCase is a derived class from unittest.TestCase.
"""

import os
import random
import shutil
import string
import tempfile
import unittest


class Modified_TestCase(unittest.TestCase):  # pylint: disable=invalid-name
    """
    <Purpose>
      Provide additional test-setup methods to make testing
      of module's methods-under-test as independent as possible.

      If you want to modify setUp()/tearDown() do:
      class Your_Test_Class(modified_TestCase):
        def setUp():
          your setup modification
          your setup modification
          ...
          modified_TestCase.setUp(self)

    <Methods>
      make_temp_directory(self, directory=None):
        Creates and returns an absolute path of a temporary directory.

      make_temp_file(self, suffix='.txt', directory=None):
        Creates and returns an absolute path of an empty temp file.

      make_temp_data_file(self, suffix='', directory=None, data = junk_data):
        Returns an absolute path of a temp file containing some data.

      random_path(self, length = 7):
        Generate a 'random' path consisting of n-length strings of random chars.


      Static Methods:
      --------------
      Following methods are static because they technically don't operate
      on any instances of the class, what they do is: they modify class variables
      (dictionaries) that are shared among all instances of the class.  So
      it is possible to call them without instantiating the class.

      random_string(length=7):
        Generate a 'length' long string of random characters.
    """

    def setUp(self):
        self._cleanup = []

    def tearDown(self):
        for cleanup_function in self._cleanup:
            # Perform clean up by executing clean-up functions.
            try:
                # OSError will occur if the directory was already removed.
                cleanup_function()

            except OSError:
                pass

    def make_temp_directory(self, directory=None):
        """Creates and returns an absolute path of a directory."""
        prefix = self.__class__.__name__ + "_"
        temp_directory = tempfile.mkdtemp(prefix=prefix, dir=directory)

        def _destroy_temp_directory():
            shutil.rmtree(temp_directory)

        self._cleanup.append(_destroy_temp_directory)
        return temp_directory

    def make_temp_file(self, suffix=".txt", directory=None):
        """Creates and returns an absolute path of an empty file."""

        prefix = "tmp_file_" + self.__class__.__name__ + "_"
        temp_file = tempfile.mkstemp(
            suffix=suffix, prefix=prefix, dir=directory
        )

        def _destroy_temp_file():
            os.unlink(temp_file[1])

        self._cleanup.append(_destroy_temp_file)
        return temp_file[1]

    def make_temp_data_file(self, suffix="", directory=None, data="junk data"):
        """Returns an absolute path of a temp file containing data."""

        temp_file_path = self.make_temp_file(suffix=suffix, directory=directory)
        temp_file = (
            open(  # pylint: disable=unspecified-encoding,consider-using-with
                temp_file_path, "wt"
            )
        )
        temp_file.write(data)
        temp_file.close()

        return temp_file_path

    def random_path(self, length=7):
        """Generate a 'random' path consisting of random n-length strings."""

        rand_path = "/" + self.random_string(length)
        for _ in range(2):
            rand_path = os.path.join(rand_path, self.random_string(length))

        return rand_path

    @staticmethod
    def random_string(length=15):
        """Generate a random string of specified length."""

        rand_str = ""
        for _ in range(length):
            rand_str += random.choice("abcdefABCDEF" + string.digits)  # nosec

        return rand_str



#!/usr/bin/env python

"""
<Program Name>
  test_formats.py

<Author>
  Vladimir Diaz <vladimir.v.diaz@gmail.com>

<Started>
  January 2017 (modified from TUF's original formats.py)

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Unit test for 'formats.py'
"""

import datetime
import unittest

import securesystemslib.formats
import securesystemslib.schema


class TestFormats(unittest.TestCase):  # pylint: disable=missing-class-docstring
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_schemas(self):
        # Test conditions for valid schemas.
        valid_schemas = {
            "ISO8601_DATETIME_SCHEMA": (
                securesystemslib.formats.ISO8601_DATETIME_SCHEMA,
                "1985-10-21T13:20:00Z",
            ),
            "UNIX_TIMESTAMP_SCHEMA": (
                securesystemslib.formats.UNIX_TIMESTAMP_SCHEMA,
                499137720,
            ),
            "HASH_SCHEMA": (
                securesystemslib.formats.HASH_SCHEMA,
                "A4582BCF323BCEF",
            ),
            "HASHDICT_SCHEMA": (
                securesystemslib.formats.HASHDICT_SCHEMA,
                {"sha256": "A4582BCF323BCEF"},
            ),
            "HEX_SCHEMA": (
                securesystemslib.formats.HEX_SCHEMA,
                "A4582BCF323BCEF",
            ),
            "KEYID_SCHEMA": (
                securesystemslib.formats.KEYID_SCHEMA,
                "123456789abcdef",
            ),
            "KEYIDS_SCHEMA": (
                securesystemslib.formats.KEYIDS_SCHEMA,
                ["123456789abcdef", "123456789abcdef"],
            ),
            "SCHEME_SCHEMA": (
                securesystemslib.formats.SCHEME_SCHEMA,
                "ecdsa-sha2-nistp256",
            ),
            "PATH_SCHEMA": (
                securesystemslib.formats.PATH_SCHEMA,
                "/home/someuser/",
            ),
            "PATHS_SCHEMA": (
                securesystemslib.formats.PATHS_SCHEMA,
                ["/home/McFly/", "/home/Tannen/"],
            ),
            "URL_SCHEMA": (
                securesystemslib.formats.URL_SCHEMA,
                "https://www.updateframework.com/",
            ),
            "NAME_SCHEMA": (
                securesystemslib.formats.NAME_SCHEMA,
                "Marty McFly",
            ),
            "TEXT_SCHEMA": (securesystemslib.formats.TEXT_SCHEMA, "Password: "),
            "BOOLEAN_SCHEMA": (securesystemslib.formats.BOOLEAN_SCHEMA, True),
            "RSAKEYBITS_SCHEMA": (
                securesystemslib.formats.RSAKEYBITS_SCHEMA,
                4096,
            ),
            "PASSWORD_SCHEMA": (
                securesystemslib.formats.PASSWORD_SCHEMA,
                "secret",
            ),
            "PASSWORDS_SCHEMA": (
                securesystemslib.formats.PASSWORDS_SCHEMA,
                ["pass1", "pass2"],
            ),
            "KEYVAL_SCHEMA": (
                securesystemslib.formats.KEYVAL_SCHEMA,
                {"public": "pubkey", "private": "privkey"},
            ),
            "PUBLIC_KEYVAL_SCHEMA": (
                securesystemslib.formats.PUBLIC_KEYVAL_SCHEMA,
                {"public": "pubkey"},
            ),
            "PUBLIC_KEYVAL_SCHEMA2": (
                securesystemslib.formats.PUBLIC_KEYVAL_SCHEMA,
                {"public": "pubkey", "private": ""},
            ),
            "RSA_SCHEME_SCHEMA_RSASSA_PSS": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsassa-pss-md5",
            ),
            "RSA_SCHEME_SCHEMA_RSASSA_PSS_2": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsassa-pss-sha1",
            ),
            "RSA_SCHEME_SCHEMA_RSASSA_PSS_3": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsassa-pss-sha224",
            ),
            "RSA_SCHEME_SCHEMA_RSASSA_PSS_4": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsassa-pss-sha256",
            ),
            "RSA_SCHEME_SCHEMA_RSASSA_PSS_5": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsassa-pss-sha384",
            ),
            "RSA_SCHEME_SCHEMA_RSASSA_PSS_6": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsassa-pss-sha512",
            ),
            "RSA_SCHEME_SCHEMA_PKCS1v15": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsa-pkcs1v15-md5",
            ),
            "RSA_SCHEME_SCHEMA_PKCS1v15_2": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsa-pkcs1v15-sha1",
            ),
            "RSA_SCHEME_SCHEMA_PKCS1v15_3": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsa-pkcs1v15-sha224",
            ),
            "RSA_SCHEME_SCHEMA_PKCS1v15_4": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsa-pkcs1v15-sha256",
            ),
            "RSA_SCHEME_SCHEMA_PKCS1v15_5": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsa-pkcs1v15-sha384",
            ),
            "RSA_SCHEME_SCHEMA_PKCS1v15_6": (
                securesystemslib.formats.RSA_SCHEME_SCHEMA,
                "rsa-pkcs1v15-sha512",
            ),
            "KEY_SCHEMA": (
                securesystemslib.formats.KEY_SCHEMA,
                {
                    "keytype": "rsa",
                    "scheme": "rsassa-pss-sha256",
                    "keyval": {"public": "pubkey", "private": "privkey"},
                },
            ),
            "PUBLIC_KEY_SCHEMA": (
                securesystemslib.formats.KEY_SCHEMA,
                {
                    "keytype": "rsa",
                    "scheme": "rsassa-pss-sha256",
                    "keyval": {"public": "pubkey"},
                },
            ),
            "PUBLIC_KEY_SCHEMA2": (
                securesystemslib.formats.KEY_SCHEMA,
                {
                    "keytype": "rsa",
                    "scheme": "rsassa-pss-sha256",
                    "keyval": {"public": "pubkey", "private": ""},
                },
            ),
            "RSAKEY_SCHEMA": (
                securesystemslib.formats.RSAKEY_SCHEMA,
                {
                    "keytype": "rsa",
                    "scheme": "rsassa-pss-sha256",
                    "keyid": "123456789abcdef",
                    "keyval": {"public": "pubkey", "private": "privkey"},
                },
            ),
            "SIGNATURE_SCHEMA": (
                securesystemslib.formats.SIGNATURE_SCHEMA,
                {"keyid": "123abc", "method": "evp", "sig": "A4582BCF323BCEF"},
            ),
            "SIGNABLE_SCHEMA": (
                securesystemslib.formats.SIGNABLE_SCHEMA,
                {
                    "signed": "signer",
                    "signatures": [
                        {
                            "keyid": "123abc",
                            "method": "evp",
                            "sig": "A4582BCF323BCEF",
                        }
                    ],
                },
            ),
            "ANY_KEYDICT_SCHEMA": (
                securesystemslib.formats.ANY_KEYDICT_SCHEMA,
                {
                    "123abc": {
                        "keytype": "rsa",
                        "scheme": "rsassa-pss-sha256",
                        "keyval": {"public": "pubkey", "private": "privkey"},
                    }
                },
            ),
        }

        # Iterate 'valid_schemas', ensuring each 'valid_schema' correctly matches
        # its respective 'schema_type'.
        for (schema_type, valid_schema) in valid_schemas.values():
            if not schema_type.matches(  # pylint: disable=no-member
                valid_schema
            ):
                print("bad schema: " + repr(valid_schema))

            self.assertEqual(
                True,
                schema_type.matches(valid_schema),  # pylint: disable=no-member
            )

        # Test conditions for invalid schemas.
        # Set the 'valid_schema' of 'valid_schemas' to an invalid
        # value and test that it does not match 'schema_type'.
        for (schema_type, valid_schema) in valid_schemas.values():
            invalid_schema = 0xBAD

            if isinstance(schema_type, securesystemslib.schema.Integer):
                invalid_schema = "BAD"
            self.assertEqual(
                False,
                schema_type.matches(  # pylint: disable=no-member
                    invalid_schema
                ),
            )

    def test_unix_timestamp_to_datetime(self):
        # Test conditions for valid arguments.
        self.assertTrue(
            datetime.datetime,
            securesystemslib.formats.unix_timestamp_to_datetime(499137720),
        )
        datetime_object = datetime.datetime(1985, 10, 26, 1, 22)
        self.assertEqual(
            datetime_object,
            securesystemslib.formats.unix_timestamp_to_datetime(499137720),
        )

        # Test conditions for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.unix_timestamp_to_datetime,
            "bad",
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.unix_timestamp_to_datetime,
            1000000000000000000000,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.unix_timestamp_to_datetime,
            -1,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.unix_timestamp_to_datetime,
            ["5"],
        )

    def test_datetime_to_unix_timestamp(self):
        # Test conditions for valid arguments.
        datetime_object = datetime.datetime(2015, 10, 21, 19, 28)
        self.assertEqual(
            1445455680,
            securesystemslib.formats.datetime_to_unix_timestamp(
                datetime_object
            ),
        )

        # Test conditions for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.datetime_to_unix_timestamp,
            "bad",
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.datetime_to_unix_timestamp,
            1000000000000000000000,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.datetime_to_unix_timestamp,
            ["1"],
        )

    def test_format_base64(self):
        # Test conditions for valid arguments.
        data = "updateframework".encode("utf-8")
        self.assertEqual(
            "dXBkYXRlZnJhbWV3b3Jr", securesystemslib.formats.format_base64(data)
        )
        self.assertTrue(
            isinstance(securesystemslib.formats.format_base64(data), str)
        )

        # Test conditions for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.format_base64,
            123,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.format_base64,
            True,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.format_base64,
            ["123"],
        )

    def test_parse_base64(self):
        # Test conditions for valid arguments.
        base64 = "dXBkYXRlZnJhbWV3b3Jr"
        self.assertEqual(
            b"updateframework", securesystemslib.formats.parse_base64(base64)
        )
        self.assertTrue(
            isinstance(securesystemslib.formats.parse_base64(base64), bytes)
        )

        # Test conditions for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.parse_base64,
            123,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.parse_base64,
            True,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.parse_base64,
            ["123"],
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.formats.parse_base64,
            "/",
        )

    def test_encode_canonical(self):
        # Test conditions for valid arguments.
        encode = securesystemslib.formats.encode_canonical
        result = []
        output = result.append

        self.assertEqual('""', encode(""))
        self.assertEqual("[1,2,3]", encode([1, 2, 3]))
        self.assertEqual("[1,2,3]", encode([1, 2, 3]))
        self.assertEqual("[]", encode([]))
        self.assertEqual("{}", encode({}))
        self.assertEqual('{"A":[99]}', encode({"A": [99]}))
        self.assertEqual('{"A":true}', encode({"A": True}))
        self.assertEqual('{"B":false}', encode({"B": False}))
        self.assertEqual('{"x":3,"y":2}', encode({"x": 3, "y": 2}))

        self.assertEqual('{"x":3,"y":null}', encode({"x": 3, "y": None}))

        # Test condition with escaping " and \
        self.assertEqual('"\\""', encode('"'))
        self.assertEqual('"\\\\"', encode("\\"))
        self.assertEqual('"\\\\\\""', encode('\\"'))

        # Condition where 'encode()' sends the result to the callable
        # 'output'.
        self.assertEqual(None, encode([1, 2, 3], output))
        self.assertEqual("[1,2,3]", "".join(result))

        # Test conditions for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            encode,
            securesystemslib.exceptions.FormatError,
        )
        self.assertRaises(securesystemslib.exceptions.FormatError, encode, 8.0)
        self.assertRaises(
            securesystemslib.exceptions.FormatError, encode, {"x": 8.0}
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, encode, 8.0, output
        )

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            encode,
            {"x": securesystemslib.exceptions.FormatError},
        )


# Run unit test.
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
<Program Name>
  test_exceptions.py

<Author>
  Vladimir Diaz

<Started>
  December 20, 2016.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Test cases for exceptions.py (mainly the exceptions defined there).
"""

import logging
import unittest

import securesystemslib.exceptions

logger = logging.getLogger(__name__)


class TestExceptions(
    unittest.TestCase
):  # pylint: disable=missing-class-docstring
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_bad_signature_error(self):
        bad_signature_error = securesystemslib.exceptions.BadSignatureError(
            "bad sig"
        )
        logger.error(bad_signature_error)

    def test_bad_hash_error(self):
        bad_hash_error = securesystemslib.exceptions.BadHashError(
            "01234", "56789"
        )
        logger.error(bad_hash_error)

    def test_invalid_metadata_json_error(self):
        format_error = securesystemslib.exceptions.FormatError(
            "Improperly formatted JSON"
        )
        invalid_metadata_json_error = (
            securesystemslib.exceptions.InvalidMetadataJSONError(format_error)
        )
        logger.error(invalid_metadata_json_error)


# Run the unit tests.
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
<Program Name>
  test_process.py

<Author>
  Lukas Puehringer <lukas.puehringer@nyu.edu>

<Started>
  Oct 4, 2018

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Test subprocess interface.

"""
import io
import os
import shlex
import sys
import tempfile
import unittest

import securesystemslib.process
import securesystemslib.settings


class Test_Process(unittest.TestCase):  # pylint: disable=invalid-name
    """Test subprocess interface."""

    def test_run_input_vs_stdin(self):
        """Test that stdin kwarg is only used if input kwarg is not supplied."""

        # Create a temporary file, passed as `stdin` argument
        fd, path = tempfile.mkstemp(text=True)
        os.write(fd, b"use stdin kwarg")
        os.close(fd)

        stdin_file = (
            open(  # pylint: disable=unspecified-encoding,consider-using-with
                path
            )
        )
        cmd = (
            sys.executable
            + " -c \"import sys; assert(sys.stdin.read() == '{}')\""
        )

        # input is used in favor of stdin
        securesystemslib.process.run(
            cmd.format("use input kwarg"),
            input=b"use input kwarg",
            stdin=stdin_file,
        )

        # stdin is only used if input is not supplied
        securesystemslib.process.run(
            cmd.format("use stdin kwarg"), stdin=stdin_file
        )

        # Clean up
        stdin_file.close()
        os.remove(path)

    def test_run_duplicate_streams(self):
        """Test output as streams and as returned."""
        # Command that prints 'foo' to stdout and 'bar' to stderr.
        cmd = (
            sys.executable + ' -c "'
            "import sys;"
            "sys.stdout.write('foo');"
            "sys.stderr.write('bar');\""
        )

        # Create and open fake targets for standard streams
        stdout_fd, stdout_fn = tempfile.mkstemp()
        stderr_fd, stderr_fn = tempfile.mkstemp()
        with io.open(  # pylint: disable=unspecified-encoding
            stdout_fn, "r"
        ) as fake_stdout_reader, os.fdopen(  # pylint: disable=unspecified-encoding
            stdout_fd, "w"
        ) as fake_stdout_writer, io.open(  # pylint: disable=unspecified-encoding
            stderr_fn, "r"
        ) as fake_stderr_reader, os.fdopen(  # pylint: disable=unspecified-encoding
            stderr_fd, "w"
        ) as fake_stderr_writer:

            # Backup original standard streams and redirect to fake targets
            real_stdout = sys.stdout
            real_stderr = sys.stderr
            sys.stdout = fake_stdout_writer
            sys.stderr = fake_stderr_writer

            # Run command
            (
                ret_code,
                ret_stdout,
                ret_stderr,
            ) = securesystemslib.process.run_duplicate_streams(cmd)

            # Rewind fake standard streams
            fake_stdout_reader.seek(0)
            fake_stderr_reader.seek(0)

            # Assert that what was printed and what was returned is correct
            self.assertTrue(ret_stdout == fake_stdout_reader.read() == "foo")
            self.assertTrue(ret_stderr == fake_stderr_reader.read() == "bar")
            # Also assert the default return value
            self.assertEqual(ret_code, 0)

            # Reset original streams
            sys.stdout = real_stdout
            sys.stderr = real_stderr

        # Remove fake standard streams
        os.remove(stdout_fn)
        os.remove(stderr_fn)

    def test_run_cmd_arg_return_code(self):
        """Test command arg as string and list using return code."""
        cmd_str = sys.executable + ' -c "' "import sys;" 'sys.exit(100)"'
        cmd_list = shlex.split(cmd_str)

        for cmd in [cmd_str, cmd_list]:
            proc = securesystemslib.process.run(cmd, check=False)
            self.assertEqual(proc.returncode, 100)

            return_code, _, _ = securesystemslib.process.run_duplicate_streams(
                cmd
            )
            self.assertEqual(return_code, 100)

    def test_run_duplicate_streams_timeout(self):
        """Test raise TimeoutExpired."""
        with self.assertRaises(
            securesystemslib.process.subprocess.TimeoutExpired
        ):
            securesystemslib.process.run_duplicate_streams(
                sys.executable + ' -c "while True: pass"', timeout=-1
            )

    def test__default_timeout(self):
        """Test default timeout modification."""
        # Backup timeout and check that it is what's returned by _default_timeout()
        timeout_old = securesystemslib.settings.SUBPROCESS_TIMEOUT
        self.assertEqual(
            securesystemslib.process._default_timeout(),  # pylint: disable=protected-access
            timeout_old,
        )

        # Modify timeout and check that _default_timeout() returns the same value
        timeout_new = timeout_old + 1
        securesystemslib.settings.SUBPROCESS_TIMEOUT = timeout_new
        self.assertEqual(
            securesystemslib.process._default_timeout(),  # pylint: disable=protected-access
            timeout_new,
        )

        # Restore original timeout
        securesystemslib.settings.SUBPROCESS_TIMEOUT = timeout_old


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
<Program Name>
  test_keys.py

<Author>
  Vladimir Diaz

<Started>
  October 10, 2013.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Test cases for test_keys.py.
"""

import copy
import unittest

import securesystemslib.ecdsa_keys
import securesystemslib.exceptions
import securesystemslib.formats
import securesystemslib.keys

KEYS = securesystemslib.keys
FORMAT_ERROR_MSG = (
    "securesystemslib.exceptions.FormatError was raised!"
    + "  Check object's format."
)
DATA_STR = "SOME DATA REQUIRING AUTHENTICITY."
DATA = securesystemslib.formats.encode_canonical(DATA_STR).encode("utf-8")


class TestKeys(unittest.TestCase):  # pylint: disable=missing-class-docstring
    @classmethod
    def setUpClass(cls):
        cls.rsakey_dict = KEYS.generate_rsa_key()
        cls.ed25519key_dict = KEYS.generate_ed25519_key()
        cls.ecdsakey_dict = KEYS.generate_ecdsa_key()
        cls.sphincskey_dict = KEYS.generate_sphincs_key()

    def test_generate_rsa_key(self):
        _rsakey_dict = KEYS.generate_rsa_key()  # pylint: disable=invalid-name

        # Check if the format of the object returned by generate() corresponds
        # to RSAKEY_SCHEMA format.
        self.assertEqual(
            None,
            securesystemslib.formats.RSAKEY_SCHEMA.check_match(_rsakey_dict),
            FORMAT_ERROR_MSG,
        )

        # Passing a bit value that is <2048 to generate() - should raise
        # 'securesystemslib.exceptions.FormatError'.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, KEYS.generate_rsa_key, 555
        )

        # Passing a string instead of integer for a bit value.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.generate_rsa_key,
            "bits",
        )

        # NOTE if random bit value >=2048 (not 4096) is passed generate(bits)
        # does not raise any errors and returns a valid key.
        self.assertTrue(
            securesystemslib.formats.RSAKEY_SCHEMA.matches(
                KEYS.generate_rsa_key(2048)
            )
        )
        self.assertTrue(
            securesystemslib.formats.RSAKEY_SCHEMA.matches(
                KEYS.generate_rsa_key(4096)
            )
        )

    def test_generate_ecdsa_key(self):
        _ecdsakey_dict = (  # pylint: disable=invalid-name
            KEYS.generate_ecdsa_key()
        )

        # Check if the format of the object returned by generate_ecdsa_key()
        # corresponds to ECDSAKEY_SCHEMA format.
        self.assertEqual(
            None,
            securesystemslib.formats.ECDSAKEY_SCHEMA.check_match(
                _ecdsakey_dict
            ),
            FORMAT_ERROR_MSG,
        )

        # Passing an invalid algorithm to generate() should raise
        # 'securesystemslib.exceptions.FormatError'.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.generate_rsa_key,
            "bad_algorithm",
        )

        # Passing a string instead of integer for a bit value.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, KEYS.generate_rsa_key, 123
        )

    def test_format_keyval_to_metadata(self):
        keyvalue = self.rsakey_dict["keyval"]
        keytype = self.rsakey_dict["keytype"]
        scheme = self.rsakey_dict["scheme"]

        key_meta = KEYS.format_keyval_to_metadata(keytype, scheme, keyvalue)

        # Check if the format of the object returned by this function corresponds
        # to KEY_SCHEMA format.
        self.assertEqual(
            None,
            securesystemslib.formats.KEY_SCHEMA.check_match(key_meta),
            FORMAT_ERROR_MSG,
        )
        key_meta = KEYS.format_keyval_to_metadata(
            keytype, scheme, keyvalue, private=True
        )

        # Check if the format of the object returned by this function corresponds
        # to KEY_SCHEMA format.
        self.assertEqual(
            None,
            securesystemslib.formats.KEY_SCHEMA.check_match(key_meta),
            FORMAT_ERROR_MSG,
        )

        # Supplying a 'bad' keyvalue.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.format_keyval_to_metadata,
            "bad_keytype",
            scheme,
            keyvalue,
            private=True,
        )

        # Test for missing 'public' entry.
        public = keyvalue["public"]
        del keyvalue["public"]
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.format_keyval_to_metadata,
            keytype,
            scheme,
            keyvalue,
        )
        keyvalue["public"] = public

        # Test for missing 'private' entry.
        private = keyvalue["private"]
        del keyvalue["private"]
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.format_keyval_to_metadata,
            keytype,
            scheme,
            keyvalue,
            private=True,
        )
        keyvalue["private"] = private

    def test_format_metadata_to_key(self):
        # Copying self.rsakey_dict so that rsakey_dict remains
        # unchanged during and after this test execution.
        test_rsakey_dict = copy.copy(self.rsakey_dict)
        del test_rsakey_dict["keyid"]

        # Call format_metadata_to_key by using the default value for keyid_hash_algorithms
        rsakey_formatted, _ = KEYS.format_metadata_to_key(test_rsakey_dict)

        # Check if the format of the object returned by calling this function with
        # default hash algorithms e.g. securesystemslib.settings.HASH_ALGORITHMS corresponds
        # to RSAKEY_SCHEMA format.
        self.assertTrue(
            securesystemslib.formats.RSAKEY_SCHEMA.matches(rsakey_formatted),
            FORMAT_ERROR_MSG,
        )

        self.assertTrue(
            securesystemslib.formats.KEY_SCHEMA.matches(rsakey_formatted),
            FORMAT_ERROR_MSG,
        )

        # Call format_metadata_to_key by using custom value for keyid_hash_algorithms
        rsakey_dict_from_meta_custom, _ = KEYS.format_metadata_to_key(
            test_rsakey_dict, keyid_hash_algorithms=["sha384"]
        )

        # Check if the format of the object returned by calling this function with
        # custom hash algorithms corresponds to RSAKEY_SCHEMA format.
        self.assertTrue(
            securesystemslib.formats.RSAKEY_SCHEMA.matches(
                rsakey_dict_from_meta_custom
            ),
            FORMAT_ERROR_MSG,
        )

        self.assertTrue(
            securesystemslib.formats.KEY_SCHEMA.matches(
                rsakey_dict_from_meta_custom
            ),
            FORMAT_ERROR_MSG,
        )

        test_rsakey_dict["keyid"] = self.rsakey_dict["keyid"]

        # Supplying a wrong number of arguments.
        self.assertRaises(TypeError, KEYS.format_metadata_to_key)
        args = (test_rsakey_dict, test_rsakey_dict)
        self.assertRaises(TypeError, KEYS.format_metadata_to_key, *args)

        # Supplying a malformed argument to the function - should get FormatError
        del test_rsakey_dict["keyval"]
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.format_metadata_to_key,
            test_rsakey_dict,
        )

    def test_helper_get_keyid(self):
        keytype = self.rsakey_dict["keytype"]
        keyvalue = self.rsakey_dict["keyval"]
        scheme = self.rsakey_dict["scheme"]

        # Check format of 'keytype'.
        self.assertEqual(
            None,
            securesystemslib.formats.KEYTYPE_SCHEMA.check_match(keytype),
            FORMAT_ERROR_MSG,
        )

        # Check format of 'keyvalue'.
        self.assertEqual(
            None,
            securesystemslib.formats.KEYVAL_SCHEMA.check_match(keyvalue),
            FORMAT_ERROR_MSG,
        )

        # Check format of 'scheme'.
        self.assertEqual(
            None,
            securesystemslib.formats.RSA_SCHEME_SCHEMA.check_match(scheme),
            FORMAT_ERROR_MSG,
        )

        keyid = KEYS._get_keyid(  # pylint: disable=protected-access
            keytype, scheme, keyvalue
        )

        # Check format of 'keyid' - the output of '_get_keyid()' function.
        self.assertEqual(
            None,
            securesystemslib.formats.KEYID_SCHEMA.check_match(keyid),
            FORMAT_ERROR_MSG,
        )

    def test_create_signature(self):
        # Creating a signature for 'DATA'.
        rsa_signature = KEYS.create_signature(self.rsakey_dict, DATA)
        ed25519_signature = KEYS.create_signature(self.ed25519key_dict, DATA)
        sphincs_signature = KEYS.create_signature(self.sphincskey_dict, DATA)

        # Check format of output.
        self.assertEqual(
            None,
            securesystemslib.formats.SIGNATURE_SCHEMA.check_match(
                rsa_signature
            ),
            FORMAT_ERROR_MSG,
        )
        self.assertEqual(
            None,
            securesystemslib.formats.SIGNATURE_SCHEMA.check_match(
                ed25519_signature
            ),
            FORMAT_ERROR_MSG,
        )
        self.assertEqual(
            None,
            securesystemslib.formats.SIGNATURE_SCHEMA.check_match(
                sphincs_signature
            ),
            FORMAT_ERROR_MSG,
        )

        # Test for invalid signature scheme.
        args = (self.rsakey_dict, DATA)

        valid_scheme = self.rsakey_dict["scheme"]
        self.rsakey_dict["scheme"] = "invalid_scheme"
        self.assertRaises(
            securesystemslib.exceptions.UnsupportedAlgorithmError,
            KEYS.create_signature,
            *args
        )
        self.rsakey_dict["scheme"] = valid_scheme

        # Removing private key from 'rsakey_dict' - should raise a TypeError.
        private = self.rsakey_dict["keyval"]["private"]
        self.rsakey_dict["keyval"]["private"] = ""

        self.assertRaises(ValueError, KEYS.create_signature, *args)

        # Supplying an incorrect number of arguments.
        self.assertRaises(TypeError, KEYS.create_signature)
        self.rsakey_dict["keyval"]["private"] = private

        # Test generation of ECDSA signatures.

        # Creating a signature for 'DATA'.
        ecdsa_signature = KEYS.create_signature(self.ecdsakey_dict, DATA)

        # Check format of output.
        self.assertEqual(
            None,
            securesystemslib.formats.SIGNATURE_SCHEMA.check_match(
                ecdsa_signature
            ),
            FORMAT_ERROR_MSG,
        )

        # Removing private key from 'ecdsakey_dict' - should raise a TypeError.
        private = self.ecdsakey_dict["keyval"]["private"]
        self.ecdsakey_dict["keyval"]["private"] = ""

        args = (self.ecdsakey_dict, DATA)
        self.assertRaises(ValueError, KEYS.create_signature, *args)

        # Supplying an incorrect number of arguments.
        self.assertRaises(TypeError, KEYS.create_signature)
        self.ecdsakey_dict["keyval"]["private"] = private

    def test_verify_signature(self):  # pylint: disable=too-many-statements
        # Creating a signature of 'DATA' to be verified.
        rsa_signature = KEYS.create_signature(self.rsakey_dict, DATA)
        ed25519_signature = KEYS.create_signature(self.ed25519key_dict, DATA)
        ecdsa_signature = KEYS.create_signature(self.ecdsakey_dict, DATA)
        sphincs_signature = KEYS.create_signature(self.sphincskey_dict, DATA)

        # Verifying the 'signature' of 'DATA'.
        verified = KEYS.verify_signature(self.rsakey_dict, rsa_signature, DATA)
        self.assertTrue(verified, "Incorrect signature.")

        # Verifying the 'ed25519_signature' of 'DATA'.
        verified = KEYS.verify_signature(
            self.ed25519key_dict, ed25519_signature, DATA
        )
        self.assertTrue(verified, "Incorrect signature.")

        # Verify that an invalid ed25519 signature scheme is rejected.
        valid_scheme = self.ed25519key_dict["scheme"]
        self.ed25519key_dict["scheme"] = "invalid_scheme"
        self.assertRaises(
            securesystemslib.exceptions.UnsupportedAlgorithmError,
            KEYS.verify_signature,
            self.ed25519key_dict,
            ed25519_signature,
            DATA,
        )
        self.ed25519key_dict["scheme"] = valid_scheme

        # Verifying the 'sphincs_signature' of 'DATA'.
        verified = KEYS.verify_signature(
            self.sphincskey_dict, sphincs_signature, DATA
        )
        self.assertTrue(verified, "Incorrect signature.")

        # Verify that an invalid sphincs signature scheme is rejected.
        valid_scheme = self.sphincskey_dict["scheme"]
        self.sphincskey_dict["scheme"] = "invalid_scheme"
        self.assertRaises(
            securesystemslib.exceptions.UnsupportedAlgorithmError,
            KEYS.verify_signature,
            self.sphincskey_dict,
            sphincs_signature,
            DATA,
        )
        self.sphincskey_dict["scheme"] = valid_scheme

        # Verifying the 'ecdsa_signature' of 'DATA'.
        verified = KEYS.verify_signature(
            self.ecdsakey_dict, ecdsa_signature, DATA
        )
        self.assertTrue(verified, "Incorrect signature.")

        # Verifying the 'ecdsa_signature' of 'DATA' with an old-style key dict
        old_key_dict = self.ecdsakey_dict.copy()
        old_key_dict["keytype"] = "ecdsa-sha2-nistp256"
        verified = KEYS.verify_signature(old_key_dict, ecdsa_signature, DATA)
        self.assertTrue(verified, "Incorrect signature.")

        # Test for an invalid ecdsa signature scheme.
        valid_scheme = self.ecdsakey_dict["scheme"]
        self.ecdsakey_dict["scheme"] = "invalid_scheme"
        self.assertRaises(
            securesystemslib.exceptions.UnsupportedAlgorithmError,
            KEYS.verify_signature,
            self.ecdsakey_dict,
            ecdsa_signature,
            DATA,
        )
        self.ecdsakey_dict["scheme"] = valid_scheme

        # Testing invalid signatures. Same signature is passed, with 'DATA' being
        # different than the original 'DATA' that was used in creating the
        # 'rsa_signature'. Function should return 'False'.

        # Modifying 'DATA'.
        _DATA_STR = "1111" + DATA_STR + "1111"  # pylint: disable=invalid-name
        _DATA = securesystemslib.formats.encode_canonical(  # pylint: disable=invalid-name
            _DATA_STR
        ).encode(
            "utf-8"
        )

        # Verifying the 'signature' of modified '_DATA'.
        verified = KEYS.verify_signature(self.rsakey_dict, rsa_signature, _DATA)
        self.assertFalse(verified, "Returned 'True' on an incorrect signature.")

        verified = KEYS.verify_signature(
            self.ed25519key_dict, ed25519_signature, _DATA
        )
        self.assertFalse(verified, "Returned 'True' on an incorrect signature.")

        verified = KEYS.verify_signature(
            self.sphincskey_dict, sphincs_signature, _DATA
        )
        self.assertFalse(verified, "Returned 'True' on an incorrect signature.")

        verified = KEYS.verify_signature(
            self.ecdsakey_dict, ecdsa_signature, _DATA
        )
        self.assertFalse(verified, "Returned 'True' on an incorrect signature.")

        # Modifying 'rsakey_dict' to pass an incorrect scheme.
        valid_scheme = self.rsakey_dict["scheme"]
        self.rsakey_dict["scheme"] = "Biff"

        args = (self.rsakey_dict, rsa_signature, DATA)
        self.assertRaises(
            securesystemslib.exceptions.UnsupportedAlgorithmError,
            KEYS.verify_signature,
            *args
        )

        # Restore
        self.rsakey_dict["scheme"] = valid_scheme

        # Verify that the KEYIDS of 'key_dict' and 'signature' match.
        valid_keyid = self.rsakey_dict["keyid"] = "12345"
        self.rsakey_dict["keyid"] = "bad123"

        self.assertRaises(
            securesystemslib.exceptions.CryptoError,
            KEYS.verify_signature,
            self.rsakey_dict,
            rsa_signature,
            DATA,
        )
        self.rsakey_dict["keyid"] = valid_keyid

        # Passing incorrect number of arguments.
        self.assertRaises(TypeError, KEYS.verify_signature)

        # Verify that the pure python 'ed25519' base case (triggered if 'pynacl'
        # is unavailable) is executed in securesystemslib.keys.verify_signature().
        KEYS._ED25519_CRYPTO_LIBRARY = (  # pylint: disable=protected-access
            "invalid"
        )
        KEYS._available_crypto_libraries = [  # pylint: disable=protected-access
            "invalid"
        ]
        verified = KEYS.verify_signature(
            self.ed25519key_dict, ed25519_signature, DATA
        )
        self.assertTrue(verified, "Incorrect signature.")

        # Verify that sphincs fails if PySPX is not installed
        KEYS.sphincs_keys.SPX_AVAIL = False  # Monkey patch availability
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            KEYS.verify_signature(self.sphincskey_dict, sphincs_signature, DATA)
        KEYS.sphincs_keys.SPX_AVAIL = True

        # Verify ecdsa key with HEX encoded keyval instead of PEM encoded keyval
        ecdsa_key = KEYS.generate_ecdsa_key()
        ecdsa_key["keyval"]["public"] = "abcd"
        # sig is not important as long as keyid is the same as the one in ecdsa_key
        sig = {"keyid": ecdsa_key["keyid"], "sig": "bb"}
        with self.assertRaises(securesystemslib.exceptions.FormatError):
            KEYS.verify_signature(ecdsa_key, sig, b"data")

        # Verify ed25519 key with PEM encoded keyval instead of HEX encoded keyval
        ed25519 = KEYS.generate_ed25519_key()
        ed25519["keyval"][
            "public"
        ] = "-----BEGIN PUBLIC KEY-----\nfoo\n-----END PUBLIC KEY-----\n"
        # sig is not important as long as keyid is the same as the one in ed25519
        sig = {"keyid": ed25519["keyid"], "sig": "bb"}
        with self.assertRaises(securesystemslib.exceptions.FormatError):
            KEYS.verify_signature(ed25519, sig, b"data")

    def test_create_rsa_encrypted_pem(self):
        # Test valid arguments.
        private = self.rsakey_dict["keyval"]["private"]
        passphrase = "secret"
        scheme = "rsassa-pss-sha256"
        encrypted_pem = KEYS.create_rsa_encrypted_pem(private, passphrase)
        self.assertTrue(
            securesystemslib.formats.PEMRSA_SCHEMA.matches(encrypted_pem)
        )
        self.assertTrue(KEYS.is_pem_private(encrypted_pem))

        # Try to import the encrypted PEM file.
        rsakey = KEYS.import_rsakey_from_private_pem(
            encrypted_pem, scheme, passphrase
        )
        self.assertTrue(securesystemslib.formats.RSAKEY_SCHEMA.matches(rsakey))

        # Test improperly formatted arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.create_rsa_encrypted_pem,
            8,
            passphrase,
        )

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.create_rsa_encrypted_pem,
            private,
            8,
        )

    def test_import_rsakey_from_private_pem(self):
        # Try to import an rsakey from a valid PEM.
        private_pem = self.rsakey_dict["keyval"]["private"]
        _ = KEYS.import_rsakey_from_private_pem(private_pem)

        # Test for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_private_pem,
            123,
        )

    def test_import_rsakey_from_public_pem(self):
        # Try to import an rsakey from a public PEM.
        pem = self.rsakey_dict["keyval"]["public"]
        rsa_key = KEYS.import_rsakey_from_public_pem(pem)

        # Check if the format of the object returned by this function corresponds
        # to 'securesystemslib.formats.RSAKEY_SCHEMA' format.
        self.assertTrue(securesystemslib.formats.RSAKEY_SCHEMA.matches(rsa_key))

        # Verify whitespace is stripped.
        self.assertEqual(
            rsa_key, KEYS.import_rsakey_from_public_pem(pem + "\n")
        )

        # Supplying a 'bad_pem' argument.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_public_pem,
            "bad_pem",
        )

        # Supplying an improperly formatted PEM.
        # Strip the PEM header and footer.
        pem_header = "-----BEGIN PUBLIC KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_public_pem,
            pem[len(pem_header) :],
        )

        pem_footer = "-----END PUBLIC KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_public_pem,
            pem[: -len(pem_footer)],
        )

        # Test for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_public_pem,
            123,
        )

    def test_import_rsakey_from_pem(self):
        # Try to import an rsakey from a public PEM.
        public_pem = self.rsakey_dict["keyval"]["public"]
        private_pem = self.rsakey_dict["keyval"]["private"]
        public_rsakey = KEYS.import_rsakey_from_pem(public_pem)
        private_rsakey = KEYS.import_rsakey_from_pem(private_pem)

        # Check if the format of the object returned by this function corresponds
        # to 'securesystemslib.formats.RSAKEY_SCHEMA' format.
        self.assertTrue(
            securesystemslib.formats.RSAKEY_SCHEMA.matches(public_rsakey)
        )
        self.assertTrue(
            securesystemslib.formats.RSAKEY_SCHEMA.matches(private_rsakey)
        )

        # Verify whitespace is stripped.
        self.assertEqual(
            public_rsakey, KEYS.import_rsakey_from_pem(public_pem + "\n")
        )
        self.assertEqual(
            private_rsakey, KEYS.import_rsakey_from_pem(private_pem + "\n")
        )

        # Supplying a 'bad_pem' argument.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_pem,
            "bad_pem",
        )

        # Supplying an improperly formatted public PEM.
        # Strip the PEM header and footer.
        pem_header = "-----BEGIN PUBLIC KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_pem,
            public_pem[len(pem_header) :],
        )

        pem_footer = "-----END PUBLIC KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_pem,
            public_pem[: -len(pem_footer)],
        )

        # Supplying an improperly formatted private PEM.
        # Strip the PEM header and footer.
        pem_header = "-----BEGIN PRIVATE KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_pem,
            private_pem[len(pem_header) :],
        )

        pem_footer = "-----END PRIVATE KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_pem,
            private_pem[: -len(pem_footer)],
        )

        # Test for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_rsakey_from_pem,
            123,
        )

    def test_import_ecdsakey_from_private_pem(self):
        # Try to import an ecdsakey from a valid PEM.
        private_pem = self.ecdsakey_dict["keyval"]["private"]
        _ = KEYS.import_ecdsakey_from_private_pem(private_pem)

        # Test for an encrypted PEM.
        scheme = "ecdsa-sha2-nistp256"
        encrypted_pem = securesystemslib.ecdsa_keys.create_ecdsa_encrypted_pem(
            private_pem, "password"
        )
        _ = KEYS.import_ecdsakey_from_private_pem(
            encrypted_pem.decode("utf-8"), scheme, "password"
        )

        # Test for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_private_pem,
            123,
        )

    def test_import_ecdsakey_from_public_pem(self):
        # Try to import an ecdsakey from a public PEM.
        pem = self.ecdsakey_dict["keyval"]["public"]
        ecdsa_key = KEYS.import_ecdsakey_from_public_pem(pem)

        # Check if the format of the object returned by this function corresponds
        # to 'securesystemslib.formats.ECDSAKEY_SCHEMA' format.
        self.assertTrue(
            securesystemslib.formats.ECDSAKEY_SCHEMA.matches(ecdsa_key)
        )

        # Verify whitespace is stripped.
        self.assertEqual(
            ecdsa_key, KEYS.import_ecdsakey_from_public_pem(pem + "\n")
        )

        # Supplying a 'bad_pem' argument.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_public_pem,
            "bad_pem",
        )

        # Supplying an improperly formatted PEM.  Strip the PEM header and footer.
        pem_header = "-----BEGIN PUBLIC KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_public_pem,
            pem[len(pem_header) :],
        )

        pem_footer = "-----END PUBLIC KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_public_pem,
            pem[: -len(pem_footer)],
        )

        # Test for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_public_pem,
            123,
        )

    def test_import_ecdsakey_from_pem(self):
        # Try to import an ecdsakey from a public PEM.
        public_pem = self.ecdsakey_dict["keyval"]["public"]
        private_pem = self.ecdsakey_dict["keyval"]["private"]
        public_ecdsakey = KEYS.import_ecdsakey_from_pem(public_pem)
        private_ecdsakey = KEYS.import_ecdsakey_from_pem(private_pem)

        # Check if the format of the object returned by this function corresponds
        # to 'securesystemslib.formats.ECDSAKEY_SCHEMA' format.
        self.assertTrue(
            securesystemslib.formats.ECDSAKEY_SCHEMA.matches(public_ecdsakey)
        )
        self.assertTrue(
            securesystemslib.formats.ECDSAKEY_SCHEMA.matches(private_ecdsakey)
        )

        # Verify whitespace is stripped.
        self.assertEqual(
            public_ecdsakey, KEYS.import_ecdsakey_from_pem(public_pem + "\n")
        )
        self.assertEqual(
            private_ecdsakey, KEYS.import_ecdsakey_from_pem(private_pem + "\n")
        )

        # Supplying a 'bad_pem' argument.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_pem,
            "bad_pem",
        )

        # Supplying an improperly formatted public PEM.  Strip the PEM header and
        # footer.
        pem_header = "-----BEGIN PUBLIC KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_pem,
            public_pem[len(pem_header) :],
        )

        pem_footer = "-----END PUBLIC KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_pem,
            public_pem[: -len(pem_footer)],
        )

        # Supplying an improperly formatted private PEM.  Strip the PEM header and
        # footer.
        pem_header = "-----BEGIN EC PRIVATE KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_pem,
            private_pem[len(pem_header) :],
        )

        pem_footer = "-----END EC PRIVATE KEY-----"
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_pem,
            private_pem[: -len(pem_footer)],
        )

        # Test for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.import_ecdsakey_from_pem,
            123,
        )

    def test_decrypt_key(self):
        # Test valid arguments.
        passphrase = "secret"
        encrypted_key = KEYS.encrypt_key(self.rsakey_dict, passphrase)
        decrypted_key = KEYS.decrypt_key(encrypted_key, passphrase)

        self.assertTrue(
            securesystemslib.formats.ANYKEY_SCHEMA.matches(decrypted_key)
        )

        # Test improperly formatted arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.decrypt_key,
            8,
            passphrase,
        )

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.decrypt_key,
            encrypted_key,
            8,
        )

    def test_extract_pem(self):
        # Normal case.
        private_pem = KEYS.extract_pem(
            self.rsakey_dict["keyval"]["private"], private_pem=True
        )
        self.assertTrue(
            securesystemslib.formats.PEMRSA_SCHEMA.matches(private_pem)
        )

        public_pem = KEYS.extract_pem(
            self.rsakey_dict["keyval"]["public"], private_pem=False
        )
        self.assertTrue(
            securesystemslib.formats.PEMRSA_SCHEMA.matches(public_pem)
        )

        # Test encrypted private pem
        encrypted_private_pem = KEYS.create_rsa_encrypted_pem(private_pem, "pw")
        encrypted_private_pem_stripped = KEYS.extract_pem(
            encrypted_private_pem, private_pem=True
        )
        self.assertTrue(
            securesystemslib.formats.PEMRSA_SCHEMA.matches(
                encrypted_private_pem_stripped
            )
        )

        # Test for an invalid PEM.
        pem_header = "-----BEGIN RSA PRIVATE KEY-----"
        pem_footer = "-----END RSA PRIVATE KEY-----"

        private_header_start = private_pem.index(pem_header)
        private_footer_start = private_pem.index(
            pem_footer, private_header_start + len(pem_header)
        )

        private_missing_header = private_pem[
            private_header_start
            + len(pem_header) : private_footer_start
            + len(pem_footer)
        ]
        private_missing_footer = private_pem[
            private_header_start:private_footer_start
        ]

        pem_header = "-----BEGIN PUBLIC KEY-----"
        pem_footer = "-----END PUBLIC KEY-----"

        public_header_start = public_pem.index(pem_header)
        public_footer_start = public_pem.index(
            pem_footer, public_header_start + len(pem_header)
        )

        public_missing_header = public_pem[
            public_header_start
            + len(pem_header) : public_footer_start
            + len(pem_footer)
        ]
        public_missing_footer = public_pem[
            public_header_start:public_footer_start
        ]

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.extract_pem,
            "invalid_pem",
            private_pem=False,
        )

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.extract_pem,
            public_missing_header,
            private_pem=False,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.extract_pem,
            private_missing_header,
            private_pem=True,
        )

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.extract_pem,
            public_missing_footer,
            private_pem=False,
        )

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.extract_pem,
            private_missing_footer,
            private_pem=True,
        )

    def test_is_pem_public(self):
        # Test for a valid PEM string.
        public_pem = self.rsakey_dict["keyval"]["public"]
        self.assertTrue(KEYS.is_pem_public(public_pem))

        # Test for a valid non-public PEM string.
        private_pem = self.rsakey_dict["keyval"]["private"]
        self.assertFalse(KEYS.is_pem_public(private_pem))

        # Test for an invalid PEM string.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, KEYS.is_pem_public, 123
        )

    def test_is_pem_private(self):
        # Test for a valid PEM string.
        private_pem_rsa = self.rsakey_dict["keyval"]["private"]
        private_pem_ec = self.ecdsakey_dict["keyval"]["private"]
        encrypted_private_pem_rsa = KEYS.create_rsa_encrypted_pem(
            private_pem_rsa, "pw"
        )

        self.assertTrue(KEYS.is_pem_private(private_pem_rsa))
        self.assertTrue(KEYS.is_pem_private(private_pem_ec, "ec"))
        self.assertTrue(KEYS.is_pem_private(encrypted_private_pem_rsa))

        # Test for a valid non-private PEM string.
        public_pem = self.rsakey_dict["keyval"]["public"]
        public_pem_ec = self.ecdsakey_dict["keyval"]["public"]
        self.assertFalse(KEYS.is_pem_private(public_pem))
        self.assertFalse(KEYS.is_pem_private(public_pem_ec, "ec"))

        # Test for unsupported keytype.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            KEYS.is_pem_private,
            private_pem_rsa,
            "bad_keytype",
        )

        # Test for an invalid PEM string.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, KEYS.is_pem_private, 123
        )


# Run the unit tests.
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env/ python

"""
<Program Name>
  test_ecdsa_keys.py

<Author>
  Vladimir Diaz <vladimir.v.diaz@gmail.com>

<Started>
  November 23, 2016.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Test cases for test_ecdsa_keys.py.
"""

import unittest

import securesystemslib.ecdsa_keys
import securesystemslib.exceptions
import securesystemslib.formats
import securesystemslib.rsa_keys

public, private = securesystemslib.ecdsa_keys.generate_public_and_private()
FORMAT_ERROR_MSG = (
    "securesystemslib.exceptions.FormatError raised.  Check object's format."
)


class TestECDSA_keys(
    unittest.TestCase
):  # pylint: disable=missing-class-docstring,invalid-name
    def setUp(self):
        pass

    def test_generate_public_and_private(self):
        (
            public,  # pylint: disable=redefined-outer-name
            private,  # pylint: disable=redefined-outer-name
        ) = securesystemslib.ecdsa_keys.generate_public_and_private()

        # Check format of 'public' and 'private'.
        self.assertEqual(
            True, securesystemslib.formats.PEMECDSA_SCHEMA.matches(public)
        )
        self.assertEqual(
            True, securesystemslib.formats.PEMECDSA_SCHEMA.matches(private)
        )

        # Test for invalid argument.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.generate_public_and_private,
            "bad_algo",
        )

    def test_create_ecdsa_public_and_private_from_pem(self):
        global public  # pylint: disable=global-statement
        global private  # pylint: disable=global-statement

        # Check format of 'public' and 'private'.
        self.assertEqual(
            True, securesystemslib.formats.PEMECDSA_SCHEMA.matches(public)
        )
        self.assertEqual(
            True, securesystemslib.formats.PEMECDSA_SCHEMA.matches(private)
        )

        # Check for a valid private pem.
        (
            public,
            private,
        ) = securesystemslib.ecdsa_keys.create_ecdsa_public_and_private_from_pem(
            private
        )

        # Check for an invalid pem (non-private).
        self.assertRaises(
            securesystemslib.exceptions.CryptoError,
            securesystemslib.ecdsa_keys.create_ecdsa_public_and_private_from_pem,
            public,
        )

        # Test for invalid argument.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.create_ecdsa_public_and_private_from_pem,
            123,
        )

    def test_create_signature(self):
        global public  # pylint: disable=global-variable-not-assigned
        global private  # pylint: disable=global-variable-not-assigned
        data = b"The quick brown fox jumps over the lazy dog"
        signature, method = securesystemslib.ecdsa_keys.create_signature(
            public, private, data
        )

        # Verify format of returned values.
        self.assertEqual(
            True,
            securesystemslib.formats.ECDSASIGNATURE_SCHEMA.matches(signature),
        )

        self.assertEqual(
            True, securesystemslib.formats.NAME_SCHEMA.matches(method)
        )
        self.assertEqual("ecdsa-sha2-nistp256", method)

        # Check for improperly formatted argument.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.create_signature,
            123,
            private,
            data,
        )

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.create_signature,
            public,
            123,
            data,
        )

        # Check for invalid 'data'.
        self.assertRaises(
            securesystemslib.exceptions.CryptoError,
            securesystemslib.ecdsa_keys.create_signature,
            public,
            private,
            123,
        )

    def test_verify_signature(self):
        global public  # pylint: disable=global-variable-not-assigned
        global private  # pylint: disable=global-variable-not-assigned
        data = b"The quick brown fox jumps over the lazy dog"
        scheme = "ecdsa-sha2-nistp256"
        signature, scheme = securesystemslib.ecdsa_keys.create_signature(
            public, private, data, scheme
        )

        valid_signature = securesystemslib.ecdsa_keys.verify_signature(
            public, scheme, signature, data
        )
        self.assertEqual(True, valid_signature)

        # Generate an RSA key so that we can verify that non-ECDSA keys are
        # rejected.
        rsa_pem, _ = securesystemslib.rsa_keys.generate_rsa_public_and_private()

        # Verify that a non-ECDSA key (via the PEM argument) is rejected.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.verify_signature,
            rsa_pem,
            scheme,
            signature,
            data,
        )

        # Check for improperly formatted arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.verify_signature,
            123,
            scheme,
            signature,
            data,
        )

        # Signature method improperly formatted.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.verify_signature,
            public,
            123,
            signature,
            data,
        )

        # Invalid signature method.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.verify_signature,
            public,
            "unsupported_scheme",
            signature,
            data,
        )

        # Signature not a string.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.verify_signature,
            public,
            scheme,
            123,
            data,
        )

        # Invalid signature..
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ecdsa_keys.verify_signature,
            public,
            scheme,
            "bad_signature",
            data,
        )

        # Check for invalid signature and data.
        self.assertEqual(
            False,
            securesystemslib.ecdsa_keys.verify_signature(
                public, scheme, signature, b"123"
            ),
        )

        # Mismatched signature.
        bad_signature = b"a" * 64
        self.assertEqual(
            False,
            securesystemslib.ecdsa_keys.verify_signature(
                public, scheme, bad_signature, data
            ),
        )

        # Generated signature created with different data.
        new_signature, scheme = securesystemslib.ecdsa_keys.create_signature(
            public, private, b"mismatched data"
        )

        self.assertEqual(
            False,
            securesystemslib.ecdsa_keys.verify_signature(
                public, scheme, new_signature, data
            ),
        )


# Run the unit tests.
if __name__ == "__main__":
    unittest.main()



"""Signer implementation for OpenPGP """
from typing import Dict, Optional

import securesystemslib.gpg.functions as gpg
from securesystemslib.signer._key import Key
from securesystemslib.signer._signer import SecretsHandler, Signature, Signer


class GPGSigner(Signer):
    """OpenPGP Signer

    Runs command in ``GNUPG`` environment variable to sign, fallback commands are
    ``gpg2`` and ``gpg``.

    Supported signing schemes are: "pgp+rsa-pkcsv1.5", "pgp+dsa-fips-180-2" and
    "pgp+eddsa-ed25519", with SHA-256 hashing.


    Arguments:
        keyid: GnuPG local user signing key id. If not passed, the default key is used.
        homedir: GnuPG home directory path. If not passed, the default homedir is used.

    """

    def __init__(
        self, keyid: Optional[str] = None, homedir: Optional[str] = None
    ):
        self.keyid = keyid
        self.homedir = homedir

    @classmethod
    def from_priv_key_uri(
        cls,
        priv_key_uri: str,
        public_key: Key,
        secrets_handler: Optional[SecretsHandler] = None,
    ) -> "GPGSigner":
        raise NotImplementedError("Incompatible with private key URIs")

    @staticmethod
    def _to_gpg_sig(sig: Signature) -> Dict:
        """Helper to convert Signature -> internal gpg signature format."""
        sig_dict = sig.to_dict()
        sig_dict["signature"] = sig_dict.pop("sig")
        return sig_dict

    @staticmethod
    def _from_gpg_sig(sig_dict: Dict) -> Signature:
        """Helper to convert internal gpg signature format -> Signature."""
        sig_dict["sig"] = sig_dict.pop("signature")
        return Signature.from_dict(sig_dict)

    def sign(self, payload: bytes) -> Signature:
        """Signs payload with ``gpg``.

        Arguments:
            payload: bytes to be signed.

        Raises:
            ValueError: The gpg command failed to create a valid signature.
            OSError: the gpg command is not present or non-executable.
            securesystemslib.exceptions.UnsupportedLibraryError: The gpg
                command is not available, or the cryptography library is
                not installed.
            securesystemslib.gpg.exceptions.CommandError: The gpg command
                returned a non-zero exit code.
            securesystemslib.gpg.exceptions.KeyNotFoundError: The used gpg
                version is not fully supported.

        Returns:
            Signature.
        """
        return self._from_gpg_sig(
            gpg.create_signature(payload, self.keyid, self.homedir)
        )



#!/usr/bin/env python

"""
<Program Name>
  test_interface.py

<Author>
  Vladimir Diaz <vladimir.v.diaz@gmail.com>

<Started>
  January 5, 2017.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Unit test for 'interface.py'.
"""

import os
import shutil
import stat
import tempfile
import unittest
from unittest import mock

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.serialization import load_pem_private_key

from securesystemslib import (  # pylint: disable=wrong-import-position
    KEY_TYPE_ECDSA,
    KEY_TYPE_ED25519,
    KEY_TYPE_RSA,
)
from securesystemslib.exceptions import (  # pylint: disable=wrong-import-position
    CryptoError,
    Error,
    FormatError,
)
from securesystemslib.formats import (  # pylint: disable=wrong-import-position
    ANY_PUBKEY_DICT_SCHEMA,
    ECDSAKEY_SCHEMA,
    ED25519KEY_SCHEMA,
    PUBLIC_KEY_SCHEMA,
    RSAKEY_SCHEMA,
)
from securesystemslib.interface import (  # pylint: disable=wrong-import-position
    _generate_and_write_ecdsa_keypair,
    _generate_and_write_ed25519_keypair,
    _generate_and_write_rsa_keypair,
    generate_and_write_ecdsa_keypair,
    generate_and_write_ecdsa_keypair_with_prompt,
    generate_and_write_ed25519_keypair,
    generate_and_write_ed25519_keypair_with_prompt,
    generate_and_write_rsa_keypair,
    generate_and_write_rsa_keypair_with_prompt,
    generate_and_write_unencrypted_ecdsa_keypair,
    generate_and_write_unencrypted_ed25519_keypair,
    generate_and_write_unencrypted_rsa_keypair,
    import_ecdsa_privatekey_from_file,
    import_ecdsa_publickey_from_file,
    import_ed25519_privatekey_from_file,
    import_ed25519_publickey_from_file,
    import_privatekey_from_file,
    import_publickeys_from_file,
    import_rsa_privatekey_from_file,
    import_rsa_publickey_from_file,
)


class TestInterfaceFunctions(
    unittest.TestCase
):  # pylint: disable=missing-class-docstring
    @classmethod
    def setUpClass(cls):
        cls.test_data_dir = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "data"
        )

        cls.path_rsa = os.path.join(cls.test_data_dir, "keystore", "rsa_key")
        cls.path_ed25519 = os.path.join(
            cls.test_data_dir, "keystore", "ed25519_key"
        )
        cls.path_ecdsa = os.path.join(
            cls.test_data_dir, "keystore", "ecdsa_key"
        )
        cls.path_no_key = os.path.join(cls.test_data_dir, "keystore", "no_key")

        cls.orig_cwd = os.getcwd()

    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp(dir=self.orig_cwd)
        os.chdir(self.tmp_dir)

    def tearDown(self):
        os.chdir(self.orig_cwd)
        shutil.rmtree(self.tmp_dir)

    def test_rsa(self):  # pylint: disable=too-many-locals,too-many-statements
        """Test RSA key _generation and import interface functions."""

        # TEST: Generate default keys and import
        # Assert location and format
        fn_default = "default"
        fn_default_ret = _generate_and_write_rsa_keypair(filepath=fn_default)

        pub = import_rsa_publickey_from_file(fn_default + ".pub")
        priv = import_rsa_privatekey_from_file(fn_default)

        self.assertEqual(fn_default, fn_default_ret)
        self.assertTrue(RSAKEY_SCHEMA.matches(pub))
        self.assertTrue(PUBLIC_KEY_SCHEMA.matches(pub))
        self.assertTrue(RSAKEY_SCHEMA.matches(priv))
        # NOTE: There is no private key schema, at least check it has a value
        self.assertTrue(priv["keyval"]["private"])

        # TEST: Generate unencrypted keys with empty prompt
        # Assert importable without password
        fn_empty_prompt = "empty_prompt"
        with mock.patch(
            "securesystemslib.interface.get_password", return_value=""
        ):
            _generate_and_write_rsa_keypair(
                filepath=fn_empty_prompt, prompt=True
            )
        import_rsa_privatekey_from_file(fn_empty_prompt)

        # TEST: Generate keys with auto-filename, i.e. keyid
        # Assert filename is keyid
        fn_keyid = _generate_and_write_rsa_keypair()
        pub = import_rsa_publickey_from_file(fn_keyid + ".pub")
        priv = import_rsa_privatekey_from_file(fn_keyid)
        self.assertTrue(
            os.path.basename(fn_keyid) == pub["keyid"] == priv["keyid"]
        )

        # TEST: Generate keys with custom bits
        # Assert length
        bits = 4096
        fn_bits = "bits"
        _generate_and_write_rsa_keypair(filepath=fn_bits, bits=bits)

        priv = import_rsa_privatekey_from_file(fn_bits)
        # NOTE: Parse PEM with pyca/cryptography to get the key size property
        obj_bits = load_pem_private_key(
            priv["keyval"]["private"].encode("utf-8"),
            password=None,
            backend=default_backend(),
        )

        self.assertEqual(obj_bits.key_size, bits)

        # TEST: Generate two keypairs with encrypted private keys using ...
        pw = "pw"
        fn_encrypted = "encrypted"
        fn_prompt = "prompt"

        # ... a passed pw ...
        _generate_and_write_rsa_keypair(filepath=fn_encrypted, password=pw)
        with mock.patch(
            "securesystemslib.interface.get_password", return_value=pw
        ):
            # ... and a prompted pw.
            _generate_and_write_rsa_keypair(filepath=fn_prompt, prompt=True)

            # Assert that both private keys are importable using the prompted pw ...
            import_rsa_privatekey_from_file(fn_prompt, prompt=True)
            import_rsa_privatekey_from_file(fn_encrypted, prompt=True)

        # ... and the passed pw.
        import_rsa_privatekey_from_file(fn_prompt, password=pw)
        import_rsa_privatekey_from_file(fn_encrypted, password=pw)

        # TEST: Import existing keys with encrypted private key (test regression)
        # Assert format
        pub = import_rsa_publickey_from_file(self.path_rsa + ".pub")
        priv = import_rsa_privatekey_from_file(self.path_rsa, "password")

        self.assertTrue(RSAKEY_SCHEMA.matches(pub))
        self.assertTrue(PUBLIC_KEY_SCHEMA.matches(pub))
        self.assertTrue(RSAKEY_SCHEMA.matches(priv))
        # NOTE: There is no private key schema, at least check it has a value
        self.assertTrue(priv["keyval"]["private"])

        # TEST: Generation errors
        for idx, (kwargs, err_msg) in enumerate(
            [
                # Error on empty password
                (
                    {"password": ""},
                    "encryption password must be 1 or more characters long",
                ),
                # Error on 'password' and 'prompt=True'
                (
                    {"password": pw, "prompt": True},
                    "passing 'password' and 'prompt=True' is not allowed",
                ),
            ]
        ):

            with self.assertRaises(
                ValueError,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ) as ctx:
                _generate_and_write_rsa_keypair(**kwargs)

            self.assertEqual(
                err_msg,
                str(ctx.exception),
                "expected: '{}' got: '{}' (row {})".format(  # pylint: disable=consider-using-f-string
                    err_msg, ctx.exception, idx
                ),
            )

        # Error on bad argument format
        for idx, kwargs in enumerate(
            [
                {"bits": 1024},  # Too low
                {"bits": "not-an-int"},
                {"filepath": 123456},  # Not a string
                {"password": 123456},  # Not a string
                {"prompt": "not-a-bool"},
            ]
        ):
            with self.assertRaises(
                FormatError,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ):
                _generate_and_write_rsa_keypair(**kwargs)

        # TEST: Import errors

        # Error public key import
        err_msg = "Invalid public pem"
        with self.assertRaises(Error) as ctx:
            import_rsa_publickey_from_file(fn_default)

        self.assertTrue(
            err_msg in str(ctx.exception),
            "expected: '{}' got: '{}'".format(  # pylint: disable=consider-using-f-string
                err_msg, ctx.exception
            ),
        )

        # Error on private key import...
        for idx, (args, kwargs, err, err_msg) in enumerate(
            [
                # Error on not a private key
                (
                    [fn_default + ".pub"],
                    {},
                    CryptoError,
                    "Could not deserialize key data",
                ),
                # Error on not encrypted
                (
                    [fn_default],
                    {"password": pw},
                    CryptoError,
                    "Password was given but private key is not encrypted",
                ),
                # Error on encrypted but no pw
                (
                    [fn_encrypted],
                    {},
                    CryptoError,
                    "Password was not given but private key is encrypted",
                ),
                # Error on encrypted but empty pw passed
                (
                    [fn_encrypted],
                    {"password": ""},
                    CryptoError,
                    "Password was not given but private key is encrypted",
                ),
                # Error on encrypted but bad pw passed
                (
                    [fn_encrypted],
                    {"password": "bad pw"},
                    CryptoError,
                    "Bad decrypt. Incorrect password?",
                ),
                # Error on pw and prompt
                (
                    [fn_default],
                    {"password": pw, "prompt": True},
                    ValueError,
                    "passing 'password' and 'prompt=True' is not allowed",
                ),
            ]
        ):

            with self.assertRaises(
                err,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ) as ctx:
                import_rsa_privatekey_from_file(*args, **kwargs)

            self.assertTrue(
                err_msg in str(ctx.exception),
                "expected: '{}' got: '{}' (row {})".format(  # pylint: disable=consider-using-f-string
                    err_msg, ctx.exception, idx
                ),
            )

        # Error on encrypted but bad pw prompted
        err_msg = "Password was not given but private key is encrypted"
        with self.assertRaises(CryptoError) as ctx, mock.patch(
            "securesystemslib.interface.get_password", return_value="bad_pw"
        ):
            import_rsa_privatekey_from_file(fn_encrypted)

        self.assertTrue(
            err_msg in str(ctx.exception),
            "expected: '{}' got: '{}'".format(  # pylint: disable=consider-using-f-string
                err_msg, ctx.exception
            ),
        )

        # Error on bad argument format
        for idx, (args, kwargs) in enumerate(
            [
                ([123456], {}),  # bad path
                ([fn_default], {"scheme": 123456}),  # bad scheme
                ([fn_default], {"scheme": "bad scheme"}),  # bad scheme
            ]
        ):
            with self.assertRaises(
                FormatError,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ):
                import_rsa_publickey_from_file(*args, **kwargs)
            with self.assertRaises(
                FormatError,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ):
                import_rsa_privatekey_from_file(*args, **kwargs)

        # bad password
        with self.assertRaises(FormatError):
            import_rsa_privatekey_from_file(fn_default, password=123456)

        # bad prompt
        with self.assertRaises(FormatError):
            import_rsa_privatekey_from_file(fn_default, prompt="not-a-bool")

    def test_ed25519(
        self,
    ):  # pylint: disable=too-many-locals,too-many-statements
        """Test ed25519 key _generation and import interface functions."""

        # TEST: Generate default keys and import
        # Assert location and format
        fn_default = "default"
        fn_default_ret = _generate_and_write_ed25519_keypair(
            filepath=fn_default
        )

        pub = import_ed25519_publickey_from_file(fn_default + ".pub")
        priv = import_ed25519_privatekey_from_file(fn_default)

        self.assertEqual(fn_default, fn_default_ret)
        self.assertTrue(ED25519KEY_SCHEMA.matches(pub))
        self.assertTrue(PUBLIC_KEY_SCHEMA.matches(pub))
        self.assertTrue(ED25519KEY_SCHEMA.matches(priv))
        # NOTE: There is no private key schema, at least check it has a value
        self.assertTrue(priv["keyval"]["private"])

        # TEST: Generate unencrypted keys with empty prompt
        # Assert importable with empty prompt password and without password
        fn_empty_prompt = "empty_prompt"
        with mock.patch(
            "securesystemslib.interface.get_password", return_value=""
        ):
            _generate_and_write_ed25519_keypair(filepath=fn_empty_prompt)
            import_ed25519_privatekey_from_file(fn_empty_prompt, prompt=True)
        import_ed25519_privatekey_from_file(fn_empty_prompt)

        # TEST: Generate keys with auto-filename, i.e. keyid
        # Assert filename is keyid
        fn_keyid = _generate_and_write_ed25519_keypair()
        pub = import_ed25519_publickey_from_file(fn_keyid + ".pub")
        priv = import_ed25519_privatekey_from_file(fn_keyid)
        self.assertTrue(
            os.path.basename(fn_keyid) == pub["keyid"] == priv["keyid"]
        )

        # TEST: Generate two keypairs with encrypted private keys using ...
        pw = "pw"
        fn_encrypted = "encrypted"
        fn_prompt = "prompt"
        # ... a passed pw ...
        _generate_and_write_ed25519_keypair(filepath=fn_encrypted, password=pw)
        with mock.patch(
            "securesystemslib.interface.get_password", return_value=pw
        ):
            # ... and a prompted pw.
            _generate_and_write_ed25519_keypair(filepath=fn_prompt, prompt=True)

            # Assert that both private keys are importable using the prompted pw ...
            import_ed25519_privatekey_from_file(fn_prompt, prompt=True)
            import_ed25519_privatekey_from_file(fn_encrypted, prompt=True)

        # ... and the passed pw.
        import_ed25519_privatekey_from_file(fn_prompt, password=pw)
        import_ed25519_privatekey_from_file(fn_encrypted, password=pw)

        # TEST: Import existing keys with encrypted private key (test regression)
        # Assert format
        pub = import_ed25519_publickey_from_file(self.path_ed25519 + ".pub")
        priv = import_ed25519_privatekey_from_file(
            self.path_ed25519, "password"
        )

        self.assertTrue(PUBLIC_KEY_SCHEMA.matches(pub))
        self.assertTrue(ED25519KEY_SCHEMA.matches(pub))
        self.assertTrue(ED25519KEY_SCHEMA.matches(priv))
        # NOTE: There is no private key schema, at least check it has a value
        self.assertTrue(priv["keyval"]["private"])

        # TEST: Unexpected behavior
        # FIXME: Should 'import_ed25519_publickey_from_file' be able to import a
        # a non-encrypted ed25519 private key? I think it should not, but it is:
        priv = import_ed25519_publickey_from_file(fn_default)
        self.assertTrue(ED25519KEY_SCHEMA.matches(priv))
        self.assertTrue(priv["keyval"]["private"])

        # FIXME: Should 'import_ed25519_privatekey_from_file' be able to import a
        # an ed25519 public key? I think it should not, but it is:
        pub = import_ed25519_privatekey_from_file(fn_default + ".pub")
        self.assertTrue(PUBLIC_KEY_SCHEMA.matches(pub))

        # TEST: Generation errors
        for idx, (kwargs, err_msg) in enumerate(
            [
                # Error on empty password
                (
                    {"password": ""},
                    "encryption password must be 1 or more characters long",
                ),
                # Error on 'password' and 'prompt=True'
                (
                    {"password": pw, "prompt": True},
                    "passing 'password' and 'prompt=True' is not allowed",
                ),
            ]
        ):

            with self.assertRaises(
                ValueError,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ) as ctx:
                _generate_and_write_ed25519_keypair(**kwargs)

            self.assertEqual(
                err_msg,
                str(ctx.exception),
                "expected: '{}' got: '{}' (row {})".format(  # pylint: disable=consider-using-f-string
                    err_msg, ctx.exception, idx
                ),
            )

        # Error on bad argument format
        for idx, kwargs in enumerate(
            [
                {"filepath": 123456},  # Not a string
                {"password": 123456},  # Not a string
                {"prompt": "not-a-bool"},
            ]
        ):
            with self.assertRaises(
                FormatError,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ):
                _generate_and_write_ed25519_keypair(**kwargs)

        # TEST: Import errors
        # Error on public key import...
        for idx, (fn, err_msg) in enumerate(
            [
                # Error on invalid json (custom key format)
                (fn_encrypted, "Cannot deserialize to a Python object"),
                # Error on invalid custom key format
                (self.path_no_key, "Missing key"),
                # Error on invalid key type
                (self.path_ecdsa + ".pub", "Invalid key type loaded"),
            ]
        ):
            with self.assertRaises(
                Error,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ) as ctx:
                import_ed25519_publickey_from_file(fn)

            self.assertTrue(
                err_msg in str(ctx.exception),
                "expected: '{}' got: '{}' (row {})".format(  # pylint: disable=consider-using-f-string
                    err_msg, ctx.exception, idx
                ),
            )

        # Error on private key import...
        for idx, (args, kwargs, err, err_msg) in enumerate(
            [
                # Error on not an ed25519 private key
                (
                    [self.path_ecdsa],
                    {},
                    CryptoError,
                    "Malformed Ed25519 key JSON, possibly due to encryption, "
                    "but no password provided?",
                ),
                # Error on not encrypted
                (
                    [fn_default],
                    {"password": pw},
                    CryptoError,
                    "Invalid encrypted file.",
                ),
                # Error on encrypted but no pw
                (
                    [fn_encrypted],
                    {},
                    CryptoError,
                    "Malformed Ed25519 key JSON, possibly due to encryption, "
                    "but no password provided?",
                ),
                # Error on encrypted but empty pw
                (
                    [fn_encrypted],
                    {"password": ""},
                    CryptoError,
                    "Decryption failed.",
                ),
                # Error on encrypted but bad pw passed
                (
                    [fn_encrypted],
                    {"password": "bad pw"},
                    CryptoError,
                    "Decryption failed.",
                ),
                # Error on pw and prompt
                (
                    [fn_default],
                    {"password": pw, "prompt": True},
                    ValueError,
                    "passing 'password' and 'prompt=True' is not allowed",
                ),
            ]
        ):

            with self.assertRaises(
                err,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ) as ctx:
                import_ed25519_privatekey_from_file(*args, **kwargs)

            self.assertTrue(
                err_msg in str(ctx.exception),
                "expected: '{}' got: '{}' (row {})".format(  # pylint: disable=consider-using-f-string
                    err_msg, ctx.exception, idx
                ),
            )

        # Error on encrypted but bad pw prompted
        err_msg = (
            "Malformed Ed25519 key JSON, possibly due to encryption, "
            "but no password provided?"
        )
        with self.assertRaises(CryptoError) as ctx, mock.patch(
            "securesystemslib.interface.get_password", return_value="bad_pw"
        ):
            import_ed25519_privatekey_from_file(fn_encrypted)

        self.assertTrue(
            err_msg in str(ctx.exception),
            "expected: '{}' got: '{}'".format(  # pylint: disable=consider-using-f-string
                err_msg, ctx.exception
            ),
        )

        # Error on bad path format
        with self.assertRaises(FormatError):
            import_ed25519_publickey_from_file(123456)
        with self.assertRaises(FormatError):
            import_ed25519_privatekey_from_file(123456)

        # Error on bad password format
        with self.assertRaises(FormatError):
            import_ed25519_privatekey_from_file(fn_default, password=123456)

        # Error on bad prompt format
        with self.assertRaises(FormatError):
            import_ed25519_privatekey_from_file(fn_default, prompt="not-a-bool")

    def test_ecdsa(self):  # pylint: disable=too-many-locals,too-many-statements
        """Test ecdsa key _generation and import interface functions."""
        # TEST: Generate default keys and import
        # Assert location and format
        fn_default = "default"
        fn_default_ret = _generate_and_write_ecdsa_keypair(filepath=fn_default)

        pub = import_ecdsa_publickey_from_file(fn_default + ".pub")
        priv = import_ecdsa_privatekey_from_file(fn_default)

        self.assertEqual(fn_default, fn_default_ret)
        self.assertTrue(ECDSAKEY_SCHEMA.matches(pub))
        self.assertTrue(PUBLIC_KEY_SCHEMA.matches(pub))
        self.assertTrue(ECDSAKEY_SCHEMA.matches(priv))
        # NOTE: There is no private key schema, at least check it has a value
        self.assertTrue(priv["keyval"]["private"])

        # TEST: Generate unencrypted keys with empty prompt
        # Assert importable with empty prompt password and without password
        fn_empty_prompt = "empty_prompt"
        with mock.patch(
            "securesystemslib.interface.get_password", return_value=""
        ):
            _generate_and_write_ecdsa_keypair(filepath=fn_empty_prompt)
            import_ecdsa_privatekey_from_file(fn_empty_prompt, prompt=True)
        import_ecdsa_privatekey_from_file(fn_empty_prompt)

        # TEST: Generate keys with auto-filename, i.e. keyid
        # Assert filename is keyid
        fn_keyid = _generate_and_write_ecdsa_keypair()
        pub = import_ecdsa_publickey_from_file(fn_keyid + ".pub")
        priv = import_ecdsa_privatekey_from_file(fn_keyid)
        self.assertTrue(
            os.path.basename(fn_keyid) == pub["keyid"] == priv["keyid"]
        )

        # TEST: Generate two key pairs with encrypted private keys using ...
        pw = "pw"
        fn_encrypted = "encrypted"
        fn_prompt = "prompt"
        # ...  a passed pw ...
        _generate_and_write_ecdsa_keypair(filepath=fn_encrypted, password=pw)
        with mock.patch(
            "securesystemslib.interface.get_password", return_value=pw
        ):
            # ... and a prompted pw.
            _generate_and_write_ecdsa_keypair(filepath=fn_prompt, prompt=True)

            # Assert that both private keys are importable using the prompted pw ...
            import_ecdsa_privatekey_from_file(fn_prompt, prompt=True)
            import_ecdsa_privatekey_from_file(fn_encrypted, prompt=True)

        # ... and the passed pw.
        import_ecdsa_privatekey_from_file(fn_prompt, password=pw)
        import_ecdsa_privatekey_from_file(fn_encrypted, password=pw)

        # TEST: Import existing keys with encrypted private key (test regression)
        # Assert format
        pub = import_ecdsa_publickey_from_file(self.path_ecdsa + ".pub")
        priv = import_ecdsa_privatekey_from_file(self.path_ecdsa, "password")

        self.assertTrue(ECDSAKEY_SCHEMA.matches(pub))
        self.assertTrue(PUBLIC_KEY_SCHEMA.matches(pub))
        self.assertTrue(ECDSAKEY_SCHEMA.matches(priv))
        # NOTE: There is no private key schema, at least check it has a value
        self.assertTrue(priv["keyval"]["private"])

        # FIXME: Should 'import_ecdsa_publickey_from_file' be able to import a
        # an ed25519 public key? I think it should not, but it is:
        import_ecdsa_publickey_from_file(self.path_ed25519 + ".pub")
        self.assertTrue(ECDSAKEY_SCHEMA.matches(pub))

        # TEST: Generation errors
        for idx, (kwargs, err_msg) in enumerate(
            [
                # Error on empty password
                (
                    {"password": ""},
                    "encryption password must be 1 or more characters long",
                ),
                # Error on 'password' and 'prompt=True'
                (
                    {"password": pw, "prompt": True},
                    "passing 'password' and 'prompt=True' is not allowed",
                ),
            ]
        ):

            with self.assertRaises(
                ValueError,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ) as ctx:
                _generate_and_write_ecdsa_keypair(**kwargs)

            self.assertEqual(
                err_msg,
                str(ctx.exception),
                "expected: '{}' got: '{}' (row {})".format(  # pylint: disable=consider-using-f-string
                    err_msg, ctx.exception, idx
                ),
            )

        # Error on bad argument format
        for idx, kwargs in enumerate(
            [
                {"filepath": 123456},  # Not a string
                {"password": 123456},  # Not a string
                {"prompt": "not-a-bool"},
            ]
        ):
            with self.assertRaises(
                FormatError,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ):
                _generate_and_write_ecdsa_keypair(**kwargs)

        # TEST: Import errors

        # Error on public key import...
        for idx, (fn, err_msg) in enumerate(
            [
                # Error on invalid json (custom key format)
                (fn_encrypted, "Cannot deserialize to a Python object"),
                # Error on invalid custom key format
                (self.path_no_key, "Missing key"),
            ]
        ):
            with self.assertRaises(
                Error,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ) as ctx:
                import_ecdsa_publickey_from_file(fn)

            self.assertTrue(
                err_msg in str(ctx.exception),
                "expected: '{}' got: '{}' (row {})".format(  # pylint: disable=consider-using-f-string
                    err_msg, ctx.exception, idx
                ),
            )

        # Error on private key import...
        for idx, (args, kwargs, err, err_msg) in enumerate(
            [
                # Error on not an ecdsa private key
                (
                    [self.path_ed25519],
                    {},
                    Error,
                    "Cannot deserialize to a Python object",
                ),
                # Error on not encrypted
                (
                    [fn_default],
                    {"password": pw},
                    CryptoError,
                    "Invalid encrypted file.",
                ),
                # Error on encrypted but no pw
                (
                    [fn_encrypted],
                    {},
                    Error,
                    "Cannot deserialize to a Python object",
                ),
                # Error on encrypted but empty pw
                (
                    [fn_encrypted],
                    {"password": ""},
                    CryptoError,
                    "Decryption failed.",
                ),
                # Error on encrypted but bad pw passed
                (
                    [fn_encrypted],
                    {"password": "bad pw"},
                    CryptoError,
                    "Decryption failed.",
                ),
                # Error on pw and prompt
                (
                    [fn_default],
                    {"password": pw, "prompt": True},
                    ValueError,
                    "passing 'password' and 'prompt=True' is not allowed",
                ),
            ]
        ):

            with self.assertRaises(
                err,
                msg="(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            ) as ctx:
                import_ecdsa_privatekey_from_file(*args, **kwargs)

            self.assertTrue(
                err_msg in str(ctx.exception),
                "expected: '{}' got: '{}' (row {})".format(  # pylint: disable=consider-using-f-string
                    err_msg, ctx.exception, idx
                ),
            )

        # Error on encrypted but bad pw prompted
        err_msg = "Decryption failed"
        with self.assertRaises(CryptoError) as ctx, mock.patch(
            "securesystemslib.interface.get_password", return_value="bad_pw"
        ):
            import_ecdsa_privatekey_from_file(fn_encrypted, prompt=True)

        self.assertTrue(
            err_msg in str(ctx.exception),
            "expected: '{}' got: '{}'".format(  # pylint: disable=consider-using-f-string
                err_msg, ctx.exception
            ),
        )

        # Error on bad path format
        with self.assertRaises(FormatError):
            import_ecdsa_publickey_from_file(123456)
        with self.assertRaises(FormatError):
            import_ecdsa_privatekey_from_file(123456)

        # Error on bad password format
        with self.assertRaises(FormatError):  # bad password
            import_ecdsa_privatekey_from_file(fn_default, password=123456)

        # Error on bad prompt format
        with self.assertRaises(FormatError):
            import_ecdsa_privatekey_from_file(fn_default, prompt="not-a-bool")

    def test_generate_keypair_wrappers(self):
        """Basic tests for thin wrappers around _generate_and_write_*_keypair.
        See 'test_rsa', 'test_ed25519' and 'test_ecdsa' for more thorough key
        generation tests for each key type.

        """
        key_pw = "pw"
        expected_priv_mode = stat.S_IFREG | stat.S_IRUSR | stat.S_IWUSR
        for idx, (gen, gen_prompt, gen_plain, import_priv, schema) in enumerate(
            [
                (
                    generate_and_write_rsa_keypair,
                    generate_and_write_rsa_keypair_with_prompt,
                    generate_and_write_unencrypted_rsa_keypair,
                    import_rsa_privatekey_from_file,
                    RSAKEY_SCHEMA,
                ),
                (
                    generate_and_write_ed25519_keypair,
                    generate_and_write_ed25519_keypair_with_prompt,
                    generate_and_write_unencrypted_ed25519_keypair,
                    import_ed25519_privatekey_from_file,
                    ED25519KEY_SCHEMA,
                ),
                (
                    generate_and_write_ecdsa_keypair,
                    generate_and_write_ecdsa_keypair_with_prompt,
                    generate_and_write_unencrypted_ecdsa_keypair,
                    import_ecdsa_privatekey_from_file,
                    ECDSAKEY_SCHEMA,
                ),
            ]
        ):

            assert_msg = (
                "(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                )
            )
            # Test generate_and_write_*_keypair creates an encrypted private key
            fn_encrypted = gen(key_pw)
            priv = import_priv(fn_encrypted, key_pw)
            self.assertTrue(schema.matches(priv), assert_msg)

            # Test that encrypted private key is generated with read and write
            # permissions for user only
            self.assertEqual(os.stat(fn_encrypted).st_mode, expected_priv_mode)

            # Test generate_and_write_*_keypair errors if password is None or empty
            with self.assertRaises(FormatError, msg=assert_msg):
                fn_encrypted = gen(None)
            with self.assertRaises(ValueError, msg=assert_msg):
                fn_encrypted = gen("")

            # Test generate_and_write_*_keypair_with_prompt creates encrypted private
            # key
            with mock.patch(
                "securesystemslib.interface.get_password", return_value=key_pw
            ):
                fn_prompt = gen_prompt()
            priv = import_priv(fn_prompt, key_pw)
            self.assertTrue(schema.matches(priv), assert_msg)

            # Test generate_and_write_*_keypair_with_prompt creates unencrypted
            # private key if no password is entered
            with mock.patch(
                "securesystemslib.interface.get_password", return_value=""
            ):
                fn_empty_prompt = gen_prompt()
            priv = import_priv(fn_empty_prompt)
            self.assertTrue(schema.matches(priv), assert_msg)

            # Test generate_and_write_unencrypted_*_keypair doesn't encrypt
            fn_unencrypted = gen_plain()
            priv = import_priv(fn_unencrypted)
            self.assertTrue(schema.matches(priv), assert_msg)

            # Test that unencrypted private key is generated with read and write
            # permissions for user only
            self.assertEqual(
                os.stat(fn_unencrypted).st_mode, expected_priv_mode
            )

    def test_import_publickeys_from_file(self):
        """Test import multiple public keys with different types."""

        # Successfully import key dict with one key per supported key type
        key_dict = import_publickeys_from_file(
            [
                self.path_rsa + ".pub",
                self.path_ed25519 + ".pub",
                self.path_ecdsa + ".pub",
            ],
            [KEY_TYPE_RSA, KEY_TYPE_ED25519, KEY_TYPE_ECDSA],
        )

        ANY_PUBKEY_DICT_SCHEMA.check_match(key_dict)
        self.assertListEqual(
            sorted([key["keytype"] for key in key_dict.values()]),
            sorted([KEY_TYPE_RSA, KEY_TYPE_ED25519, KEY_TYPE_ECDSA]),
        )

        # Successfully import default rsa key
        key_dict = import_publickeys_from_file([self.path_rsa + ".pub"])
        ANY_PUBKEY_DICT_SCHEMA.check_match(key_dict)
        RSAKEY_SCHEMA.check_match(list(key_dict.values()).pop())

        # Bad default rsa key type for ed25519
        with self.assertRaises(Error):
            import_publickeys_from_file([self.path_ed25519 + ".pub"])

        # Bad ed25519 key type for rsa key
        with self.assertRaises(Error):
            import_publickeys_from_file(
                [self.path_rsa + ".pub"], [KEY_TYPE_ED25519]
            )

        # Unsupported key type
        with self.assertRaises(FormatError):
            import_publickeys_from_file(
                [self.path_ed25519 + ".pub"], ["KEY_TYPE_UNSUPPORTED"]
            )

        # Mismatching arguments lists lenghts
        with self.assertRaises(FormatError):
            import_publickeys_from_file(
                [self.path_rsa + ".pub", self.path_ed25519 + ".pub"],
                [KEY_TYPE_ED25519],
            )

    def test_import_privatekey_from_file(self):
        """Test generic private key import function."""

        pw = "password"
        for idx, (path, key_type, key_schema) in enumerate(
            [
                (self.path_rsa, None, RSAKEY_SCHEMA),  # default key type
                (self.path_rsa, KEY_TYPE_RSA, RSAKEY_SCHEMA),
                (self.path_ed25519, KEY_TYPE_ED25519, ED25519KEY_SCHEMA),
                (self.path_ecdsa, KEY_TYPE_ECDSA, ECDSAKEY_SCHEMA),
            ]
        ):

            # Successfully import key per supported type, with ...
            # ... passed password
            key = import_privatekey_from_file(
                path, key_type=key_type, password=pw
            )
            self.assertTrue(
                key_schema.matches(key),
                "(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            )

            # ... entered password on mock-prompt
            with mock.patch(
                "securesystemslib.interface.get_password", return_value=pw
            ):
                key = import_privatekey_from_file(
                    path, key_type=key_type, prompt=True
                )
            self.assertTrue(
                key_schema.matches(key),
                "(row {})".format(  # pylint: disable=consider-using-f-string
                    idx
                ),
            )

        # Error on wrong key for default key type
        with self.assertRaises(Error):
            import_privatekey_from_file(self.path_ed25519, password=pw)

        # Error on unsupported key type
        with self.assertRaises(FormatError):
            import_privatekey_from_file(
                self.path_rsa, key_type="KEY_TYPE_UNSUPPORTED", password=pw
            )


# Run the test cases.
if __name__ == "__main__":
    unittest.main()



"""
<Program Name>
  ecdsa_keys.py

<Author>
  Vladimir Diaz <vladimir.v.diaz@gmail.com>

<Started>
  November 22, 2016.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  The goal of this module is to support ECDSA keys and signatures.  ECDSA is an
  elliptic-curve digital signature algorithm.  It grants a similar level of
  security as RSA, but uses smaller keys.  No subexponential-time algorithm is
  known for the elliptic curve discrete logarithm problem.

  https://en.wikipedia.org/wiki/Elliptic_Curve_Digital_Signature_Algorithm

  'securesystemslib.ecdsa_keys.py' calls the 'cryptography' library to perform
  all of the ecdsa-related operations.

  The ecdsa-related functions included here are generate(), create_signature()
  and verify_signature().  The 'cryptography' library is used by ecdsa_keys.py
  to perform the actual ECDSA computations, and the functions listed above can
  be viewed as an easy-to-use public interface.
 """

import logging

# Import cryptography modules to support ecdsa keys and signatures.
CRYPTO = True
NO_CRYPTO_MSG = "ECDSA key support requires the cryptography library"
try:
    from cryptography.exceptions import InvalidSignature, UnsupportedAlgorithm
    from cryptography.hazmat.backends import default_backend
    from cryptography.hazmat.primitives import hashes, serialization
    from cryptography.hazmat.primitives.asymmetric import ec
    from cryptography.hazmat.primitives.serialization import (
        load_pem_private_key,
        load_pem_public_key,
    )

    _SCHEME_HASHER = {
        "ecdsa-sha2-nistp256": ec.ECDSA(hashes.SHA256()),
        "ecdsa-sha2-nistp384": ec.ECDSA(hashes.SHA384()),
    }

except ImportError:
    CRYPTO = False

# Perform object format-checking and add ability to handle/raise exceptions.
from securesystemslib import (  # pylint: disable=wrong-import-position
    exceptions,
    formats,
)

_SUPPORTED_ECDSA_SCHEMES = ["ecdsa-sha2-nistp256"]

logger = logging.getLogger(__name__)


def generate_public_and_private(scheme="ecdsa-sha2-nistp256"):
    """
    <Purpose>
      Generate a pair of ECDSA public and private keys with one of the supported,
      external cryptography libraries.  The public and private keys returned
      conform to 'securesystemslib.formats.PEMECDSA_SCHEMA' and
      'securesystemslib.formats.PEMECDSA_SCHEMA', respectively.

      The public ECDSA public key has the PEM format:
      TODO: should we encrypt the private keys returned here?  Should the
      create_signature() accept encrypted keys?

      '-----BEGIN PUBLIC KEY-----

      ...

      '-----END PUBLIC KEY-----'



      The private ECDSA private key has the PEM format:

      '-----BEGIN EC PRIVATE KEY-----

      ...

      -----END EC PRIVATE KEY-----'

      >>> public, private = generate_public_and_private()
      >>> securesystemslib.formats.PEMECDSA_SCHEMA.matches(public)
      True
      >>> securesystemslib.formats.PEMECDSA_SCHEMA.matches(private)
      True

    <Arguments>
      scheme:
        A string indicating which algorithm to use for the generation of the
        public and private ECDSA keys.  'ecdsa-sha2-nistp256' is the only
        currently supported ECDSA algorithm, which is supported by OpenSSH and
        specified in RFC 5656 (https://tools.ietf.org/html/rfc5656).

    <Exceptions>
      securesystemslib.exceptions.FormatError, if 'algorithm' is improperly
      formatted.

      securesystemslib.exceptions.UnsupportedAlgorithmError, if 'scheme' is an
      unsupported algorithm.

      securesystemslib.exceptions.UnsupportedLibraryError, if the cryptography
      module is not available.

    <Side Effects>
      None.

    <Returns>
      A (public, private) tuple that conform to
      'securesystemslib.formats.PEMECDSA_SCHEMA' and
      'securesystemslib.formats.PEMECDSA_SCHEMA', respectively.
    """

    if not CRYPTO:  # pragma: no cover
        raise exceptions.UnsupportedLibraryError(NO_CRYPTO_MSG)

    # Does 'scheme' have the correct format?
    # Verify that 'scheme' is of the correct type, and that it's one of the
    # supported ECDSA .  It must conform to
    # 'securesystemslib.formats.ECDSA_SCHEME_SCHEMA'.  Raise
    # 'securesystemslib.exceptions.FormatError' if the check fails.
    formats.ECDSA_SCHEME_SCHEMA.check_match(scheme)

    public_key = None
    private_key = None

    # An if-clause is strictly not needed, since 'ecdsa_sha2-nistp256' is the
    # only currently supported ECDSA signature scheme.  Nevertheness, include the
    # conditional statement to accomodate any schemes that might be added.
    if scheme == "ecdsa-sha2-nistp256":
        private_key = ec.generate_private_key(ec.SECP256R1, default_backend())
        public_key = private_key.public_key()

    # The ECDSA_SCHEME_SCHEMA.check_match() above should have detected any
    # invalid 'scheme'.  This is a defensive check.
    else:  # pragma: no cover
        raise exceptions.UnsupportedAlgorithmError(
            "An unsupported"
            " scheme specified: " + repr(scheme) + ".\n  Supported"
            " algorithms: " + repr(_SUPPORTED_ECDSA_SCHEMES)
        )

    private_pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption(),
    )

    public_pem = public_key.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo,
    )

    return public_pem.decode("utf-8"), private_pem.decode("utf-8")


def create_signature(
    public_key, private_key, data, scheme="ecdsa-sha2-nistp256"
):
    """
    <Purpose>
      Return a (signature, scheme) tuple.

      >>> requested_scheme = 'ecdsa-sha2-nistp256'
      >>> public, private = generate_public_and_private(requested_scheme)
      >>> data = b'The quick brown fox jumps over the lazy dog'
      >>> signature, scheme = create_signature(public, private, data, requested_scheme)
      >>> securesystemslib.formats.ECDSASIGNATURE_SCHEMA.matches(signature)
      True
      >>> requested_scheme == scheme
      True

    <Arguments>
      public:
        The ECDSA public key in PEM format.

      private:
        The ECDSA private key in PEM format.

      data:
        Byte data used by create_signature() to generate the signature returned.

      scheme:
        The signature scheme used to generate the signature.  For example:
        'ecdsa-sha2-nistp256'.

    <Exceptions>
      securesystemslib.exceptions.FormatError, if the arguments are improperly
      formatted.

      securesystemslib.exceptions.CryptoError, if a signature cannot be created.

      securesystemslib.exceptions.UnsupportedAlgorithmError, if 'scheme' is not
      one of the supported signature schemes.

      securesystemslib.exceptions.UnsupportedLibraryError, if the cryptography
      module is not available.

    <Side Effects>
      None.

    <Returns>
      A signature dictionary conformat to
      'securesystemslib.format.SIGNATURE_SCHEMA'.  ECDSA signatures are XX bytes,
      however, the hexlified signature is stored in the dictionary returned.
    """

    if not CRYPTO:  # pragma: no cover
        raise exceptions.UnsupportedLibraryError(NO_CRYPTO_MSG)

    # Do 'public_key' and 'private_key' have the correct format?
    # This check will ensure that the arguments conform to
    # 'securesystemslib.formats.PEMECDSA_SCHEMA'.  Raise
    # 'securesystemslib.exceptions.FormatError' if the check fails.
    formats.PEMECDSA_SCHEMA.check_match(public_key)

    # Is 'private_key' properly formatted?
    formats.PEMECDSA_SCHEMA.check_match(private_key)

    # Is 'scheme' properly formatted?
    formats.ECDSA_SCHEME_SCHEMA.check_match(scheme)

    # 'ecdsa-sha2-nistp256' is the only currently supported ECDSA scheme, so this
    # if-clause isn't strictly needed.  Nevertheless, the conditional statement
    # is included to accommodate multiple schemes that can potentially be added
    # in the future.
    if scheme == "ecdsa-sha2-nistp256":
        try:
            private_key = load_pem_private_key(
                private_key.encode("utf-8"),
                password=None,
                backend=default_backend(),
            )

            signature = private_key.sign(data, ec.ECDSA(hashes.SHA256()))

        except TypeError as e:
            raise exceptions.CryptoError(
                "Could not create" " signature: " + str(e)
            )

    # A defensive check for an invalid 'scheme'.  The
    # ECDSA_SCHEME_SCHEMA.check_match() above should have already validated it.
    else:  # pragma: no cover
        raise exceptions.UnsupportedAlgorithmError(
            "Unsupported" " signature scheme is specified: " + repr(scheme)
        )

    return signature, scheme


def verify_signature(public_key, scheme, signature, data):
    """
    <Purpose>
      Verify that 'signature' was produced by the private key associated with
      'public_key'.

      >>> scheme = 'ecdsa-sha2-nistp256'
      >>> public, private = generate_public_and_private(scheme)
      >>> data = b'The quick brown fox jumps over the lazy dog'
      >>> signature, scheme = create_signature(public, private, data, scheme)
      >>> verify_signature(public, scheme, signature, data)
      True
      >>> verify_signature(public, scheme, signature, b'bad data')
      False

    <Arguments>
      public_key:
        The ECDSA public key in PEM format.  The public key is needed to verify
        'signature'.

      scheme:
        The signature scheme used to generate 'signature'.  For example:
        'ecdsa-sha2-nistp256'.

      signature:
        The signature to be verified, which should have been generated by
        the private key associated with 'public_key'.  'data'.

      data:
        Byte data that was used by create_signature() to generate 'signature'.

    <Exceptions>
      securesystemslib.exceptions.FormatError, if any of the arguments are
      improperly formatted.

      securesystemslib.exceptions.UnsupportedAlgorithmError, if 'scheme' is
      not one of the supported signature schemes.

      securesystemslib.exceptions.UnsupportedLibraryError, if the cryptography
      module is not available.

    <Side Effects>
      None.

    <Returns>
      Boolean, indicating whether the 'signature' of data was generated by
      the private key associated with 'public_key'.
    """

    if not CRYPTO:  # pragma: no cover
        raise exceptions.UnsupportedLibraryError(NO_CRYPTO_MSG)

    # Are the arguments properly formatted?
    # If not, raise 'securesystemslib.exceptions.FormatError'.
    formats.PEMECDSA_SCHEMA.check_match(public_key)
    formats.ECDSA_SCHEME_SCHEMA.check_match(scheme)
    formats.ECDSASIGNATURE_SCHEMA.check_match(signature)

    try:
        ecdsa_key = load_pem_public_key(
            public_key.encode("utf-8"), backend=default_backend()
        )
    except ValueError as e:
        raise exceptions.FormatError(
            f"Failed to load PEM key {public_key}"
        ) from e

    if not isinstance(ecdsa_key, ec.EllipticCurvePublicKey):
        raise exceptions.FormatError(
            "Invalid ECDSA public" " key: " + repr(public_key)
        )
    logger.debug("Loaded a valid ECDSA public key.")

    # verify() raises an 'InvalidSignature' exception if 'signature'
    # is invalid.
    try:
        ecdsa_key.verify(signature, data, _SCHEME_HASHER[scheme])
        return True

    except (TypeError, InvalidSignature):
        return False


def create_ecdsa_public_and_private_from_pem(pem, password=None):
    """
    <Purpose>
      Create public and private ECDSA keys from a private 'pem'.  The public and
      private keys are strings in PEM format:

      public: '-----BEGIN PUBLIC KEY----- ... -----END PUBLIC KEY-----',
      private: '-----BEGIN EC PRIVATE KEY----- ... -----END EC PRIVATE KEY-----'}}

      >>> junk, private = generate_public_and_private()
      >>> public, private = create_ecdsa_public_and_private_from_pem(private)
      >>> securesystemslib.formats.PEMECDSA_SCHEMA.matches(public)
      True
      >>> securesystemslib.formats.PEMECDSA_SCHEMA.matches(private)
      True
      >>> passphrase = 'secret'
      >>> encrypted_pem = create_ecdsa_encrypted_pem(private, passphrase)
      >>> public, private = create_ecdsa_public_and_private_from_pem(encrypted_pem, passphrase)
      >>> securesystemslib.formats.PEMECDSA_SCHEMA.matches(public)
      True
      >>> securesystemslib.formats.PEMECDSA_SCHEMA.matches(private)
      True

    <Arguments>
      pem:
        A string in PEM format.  The private key is extracted and returned in
        an ecdsakey object.

      password: (optional)
        The password, or passphrase, to decrypt the private part of the ECDSA key
        if it is encrypted.  'password' is not used directly as the encryption
        key, a stronger encryption key is derived from it.

    <Exceptions>
      securesystemslib.exceptions.FormatError, if the arguments are improperly
      formatted.

      securesystemslib.exceptions.UnsupportedAlgorithmError, if the ECDSA key
      pair could not be extracted, possibly due to an unsupported algorithm.

      securesystemslib.exceptions.UnsupportedLibraryError, if the cryptography
      module is not available.

    <Side Effects>
      None.

    <Returns>
      A dictionary containing the ECDSA keys and other identifying information.
      Conforms to 'securesystemslib.formats.ECDSAKEY_SCHEMA'.
    """

    if not CRYPTO:  # pragma: no cover
        raise exceptions.UnsupportedLibraryError(NO_CRYPTO_MSG)

    # Does 'pem' have the correct format?
    # This check will ensure 'pem' conforms to
    # 'securesystemslib.formats.ECDSARSA_SCHEMA'.
    formats.PEMECDSA_SCHEMA.check_match(pem)

    if password is not None:
        formats.PASSWORD_SCHEMA.check_match(password)
        password = password.encode("utf-8")

    else:
        logger.debug(
            "The password/passphrase is unset.  The PEM is expected"
            " to be unencrypted."
        )

    public = None
    private = None

    # Generate the public and private ECDSA keys.  The pyca/cryptography library
    # performs the actual import operation.
    try:
        private = load_pem_private_key(
            pem.encode("utf-8"), password=password, backend=default_backend()
        )

    except (ValueError, UnsupportedAlgorithm) as e:
        raise exceptions.CryptoError(
            "Could not import private" " PEM.\n" + str(e)
        )

    public = private.public_key()

    # Serialize public and private keys to PEM format.
    private = private.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption(),
    )

    public = public.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo,
    )

    return public.decode("utf-8"), private.decode("utf-8")


def create_ecdsa_encrypted_pem(private_pem, passphrase):
    """
    <Purpose>
      Return a string in PEM format, where the private part of the ECDSA key is
      encrypted. The private part of the ECDSA key is encrypted as done by
      pyca/cryptography: "Encrypt using the best available encryption for a given
      key's backend. This is a curated encryption choice and the algorithm may
      change over time."

      >>> junk, private = generate_public_and_private()
      >>> passphrase = 'secret'
      >>> encrypted_pem = create_ecdsa_encrypted_pem(private, passphrase)
      >>> securesystemslib.formats.PEMECDSA_SCHEMA.matches(encrypted_pem)
      True

    <Arguments>
      private_pem:
      The private ECDSA key string in PEM format.

      passphrase:
      The passphrase, or password, to encrypt the private part of the ECDSA
      key. 'passphrase' is not used directly as the encryption key, a stronger
      encryption key is derived from it.

      <Exceptions>
        securesystemslib.exceptions.FormatError, if the arguments are improperly
        formatted.

        securesystemslib.exceptions.CryptoError, if an ECDSA key in encrypted PEM
        format cannot be created.

      securesystemslib.exceptions.UnsupportedLibraryError, if the cryptography
      module is not available.

    <Side Effects>
      None.

    <Returns>
      A string in PEM format, where the private RSA portion is encrypted.
      Conforms to 'securesystemslib.formats.PEMECDSA_SCHEMA'.
    """

    if not CRYPTO:  # pragma: no cover
        raise exceptions.UnsupportedLibraryError(NO_CRYPTO_MSG)

    # Does 'private_key' have the correct format?
    # Raise 'securesystemslib.exceptions.FormatError' if the check fails.
    formats.PEMRSA_SCHEMA.check_match(private_pem)

    # Does 'passphrase' have the correct format?
    formats.PASSWORD_SCHEMA.check_match(passphrase)

    private = load_pem_private_key(
        private_pem.encode("utf-8"), password=None, backend=default_backend()
    )

    encrypted_private_pem = private.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.BestAvailableEncryption(
            passphrase.encode("utf-8")
        ),
    )

    return encrypted_private_pem


if __name__ == "__main__":
    # The interactive sessions of the documentation strings can
    # be tested by running 'ecdsa_keys.py' as a standalone module.
    # python -B ecdsa_keys.py
    import doctest

    doctest.testmod()



"""
<Program Name>
  exceptions.py

<Author>
  Geremy Condra
  Vladimir Diaz <vladimir.v.diaz@gmail.com>

<Started>
  VD: April 4, 2012 Revision.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Define exceptions.  The names chosen for exception classes should end in
  'Error' (except where there is a good reason not to).
"""


class Error(Exception):
    """Indicate a generic error."""


class Warning(Warning):  # pylint: disable=redefined-builtin
    """Generic warning.  It is used by the 'warnings' module."""


class FormatError(Error):
    """Indicate an error while validating an object's format."""


class InvalidMetadataJSONError(FormatError):
    """Indicate that a metadata file is not valid JSON."""

    def __init__(self, exception):  # pylint: disable=super-init-not-called
        # Store the original exception.
        self.exception = exception

    def __str__(self):
        # Show the original exception.
        return repr(self.exception)


class UnsupportedAlgorithmError(Error):
    """Indicate an error while trying to identify a user-specified algorithm."""


class BadHashError(Error):
    """Indicate an error while checking the value a hash object."""

    def __init__(
        self, expected_hash, observed_hash
    ):  # pylint: disable=super-init-not-called
        self.expected_hash = expected_hash
        self.observed_hash = observed_hash

    def __str__(self):
        return (
            "Observed hash ("
            + repr(self.observed_hash)
            + ") != expected hash ("
            + repr(self.expected_hash)
            + ")"
        )


class BadPasswordError(Error):
    """Indicate an error after encountering an invalid password."""


class CryptoError(Error):
    """Indicate any cryptography-related errors."""


class BadSignatureError(CryptoError):
    """Indicate that some metadata has a bad signature."""

    def __init__(
        self, metadata_role_name
    ):  # pylint: disable=super-init-not-called
        self.metadata_role_name = metadata_role_name

    def __str__(self):
        return repr(self.metadata_role_name) + " metadata has bad signature."


class UnknownMethodError(CryptoError):
    """Indicate that a user-specified cryptograpthic method is unknown."""


class UnsupportedLibraryError(Error):
    """Indicate that a supported library could not be located or imported."""


class InvalidNameError(Error):
    """Indicate an error while trying to validate any type of named object."""


class NotFoundError(Error):
    """If a required configuration or resource is not found."""


class URLMatchesNoPatternError(Error):
    """If a URL does not match a user-specified regular expression."""


class InvalidConfigurationError(Error):
    """If a configuration object does not match the expected format."""


class StorageError(Error):
    """Indicate an error occured during interaction with an abstracted storage
    backend."""


class UnverifiedSignatureError(Error):
    """Signature could not be verified: either signature was incorrect or
    something failed during process (see VerificationError)"""


class VerificationError(UnverifiedSignatureError):
    """Signature could not be verified because something failed in the process"""



"""
The Signer API

This module provides extensible interfaces for public keys and signers:
Some implementations are provided by default but more can be added by users.
"""
from securesystemslib.signer._gcp_signer import GCPSigner
from securesystemslib.signer._gpg_signer import GPGSigner
from securesystemslib.signer._hsm_signer import HSMSigner
from securesystemslib.signer._key import KEY_FOR_TYPE_AND_SCHEME, Key, SSlibKey
from securesystemslib.signer._signature import Signature
from securesystemslib.signer._signer import (
    SIGNER_FOR_URI_SCHEME,
    SecretsHandler,
    Signer,
    SSlibSigner,
)

# Register supported private key uri schemes and the Signers implementing them
SIGNER_FOR_URI_SCHEME.update(
    {
        SSlibSigner.ENVVAR_URI_SCHEME: SSlibSigner,
        SSlibSigner.FILE_URI_SCHEME: SSlibSigner,
        GCPSigner.SCHEME: GCPSigner,
        HSMSigner.SCHEME: HSMSigner,
    }
)

# Register supported key types and schemes, and the Keys implementing them
KEY_FOR_TYPE_AND_SCHEME.update(
    {
        ("ecdsa", "ecdsa-sha2-nistp256"): SSlibKey,
        ("ecdsa", "ecdsa-sha2-nistp384"): SSlibKey,
        ("ecdsa-sha2-nistp256", "ecdsa-sha2-nistp256"): SSlibKey,
        ("ecdsa-sha2-nistp384", "ecdsa-sha2-nistp384"): SSlibKey,
        ("ed25519", "ed25519"): SSlibKey,
        ("rsa", "rsassa-pss-md5"): SSlibKey,
        ("rsa", "rsassa-pss-sha1"): SSlibKey,
        ("rsa", "rsassa-pss-sha224"): SSlibKey,
        ("rsa", "rsassa-pss-sha256"): SSlibKey,
        ("rsa", "rsassa-pss-sha384"): SSlibKey,
        ("rsa", "rsassa-pss-sha512"): SSlibKey,
        ("rsa", "rsa-pkcs1v15-md5"): SSlibKey,
        ("rsa", "rsa-pkcs1v15-sha1"): SSlibKey,
        ("rsa", "rsa-pkcs1v15-sha224"): SSlibKey,
        ("rsa", "rsa-pkcs1v15-sha256"): SSlibKey,
        ("rsa", "rsa-pkcs1v15-sha384"): SSlibKey,
        ("rsa", "rsa-pkcs1v15-sha512"): SSlibKey,
        ("sphincs", "sphincs-shake-128s"): SSlibKey,
    }
)



#!/usr/bin/env/ python

"""
<Program Name>
  test_ed25519_keys.py

<Author>
  Vladimir Diaz <vladimir.v.diaz@gmail.com>

<Started>
  October 11, 2013.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Test cases for test_ed25519_keys.py.
"""

import os
import unittest

import securesystemslib.ed25519_keys
import securesystemslib.exceptions
import securesystemslib.formats

public, private = securesystemslib.ed25519_keys.generate_public_and_private()
FORMAT_ERROR_MSG = (
    "securesystemslib.exceptions.FormatError raised.  Check object's format."
)


class TestEd25519_keys(
    unittest.TestCase
):  # pylint: disable=missing-class-docstring,invalid-name
    def setUp(self):
        pass

    def test_generate_public_and_private(self):
        pub, priv = securesystemslib.ed25519_keys.generate_public_and_private()

        # Check format of 'pub' and 'priv'.
        self.assertEqual(
            True, securesystemslib.formats.ED25519PUBLIC_SCHEMA.matches(pub)
        )
        self.assertEqual(
            True, securesystemslib.formats.ED25519SEED_SCHEMA.matches(priv)
        )

    def test_create_signature(self):
        global public  # pylint: disable=global-variable-not-assigned
        global private  # pylint: disable=global-variable-not-assigned
        data = b"The quick brown fox jumps over the lazy dog"
        scheme = "ed25519"
        signature, scheme = securesystemslib.ed25519_keys.create_signature(
            public, private, data, scheme
        )

        # Verify format of returned values.
        self.assertEqual(
            True,
            securesystemslib.formats.ED25519SIGNATURE_SCHEMA.matches(signature),
        )

        self.assertEqual(
            True, securesystemslib.formats.ED25519_SIG_SCHEMA.matches(scheme)
        )
        self.assertEqual("ed25519", scheme)

        # Check for improperly formatted argument.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ed25519_keys.create_signature,
            123,
            private,
            data,
            scheme,
        )

        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ed25519_keys.create_signature,
            public,
            123,
            data,
            scheme,
        )

        # Check for invalid 'data'.
        self.assertRaises(
            securesystemslib.exceptions.CryptoError,
            securesystemslib.ed25519_keys.create_signature,
            public,
            private,
            123,
            scheme,
        )

    def test_verify_signature(self):
        global public  # pylint: disable=global-variable-not-assigned
        global private  # pylint: disable=global-variable-not-assigned
        data = b"The quick brown fox jumps over the lazy dog"
        scheme = "ed25519"
        signature, scheme = securesystemslib.ed25519_keys.create_signature(
            public, private, data, scheme
        )

        valid_signature = securesystemslib.ed25519_keys.verify_signature(
            public, scheme, signature, data
        )
        self.assertEqual(True, valid_signature)

        bad_signature = os.urandom(64)
        valid_signature = securesystemslib.ed25519_keys.verify_signature(
            public, scheme, bad_signature, data
        )
        self.assertEqual(False, valid_signature)

        # Check for improperly formatted arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ed25519_keys.verify_signature,
            123,
            scheme,
            signature,
            data,
        )

        # Signature method improperly formatted.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ed25519_keys.verify_signature,
            public,
            123,
            signature,
            data,
        )

        # Invalid signature method.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ed25519_keys.verify_signature,
            public,
            "unsupported_scheme",
            signature,
            data,
        )

        # Signature not a string.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ed25519_keys.verify_signature,
            public,
            scheme,
            123,
            data,
        )

        # Invalid signature length, which must be exactly 64 bytes..
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            securesystemslib.ed25519_keys.verify_signature,
            public,
            scheme,
            "bad_signature",
            data,
        )

        # Check for invalid signature and data.
        # Mismatched data.
        self.assertEqual(
            False,
            securesystemslib.ed25519_keys.verify_signature(
                public, scheme, signature, b"123"
            ),
        )

        # Mismatched signature.
        bad_signature = b"a" * 64
        self.assertEqual(
            False,
            securesystemslib.ed25519_keys.verify_signature(
                public, scheme, bad_signature, data
            ),
        )

        # Generated signature created with different data.
        new_signature, scheme = securesystemslib.ed25519_keys.create_signature(
            public, private, b"mismatched data", scheme
        )

        self.assertEqual(
            False,
            securesystemslib.ed25519_keys.verify_signature(
                public, scheme, new_signature, data
            ),
        )


# Run the unit tests.
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
<Program Name>
  process.py

<Author>
  Trishank Karthik Kuppusamy <trishank.kuppusamy@datadoghq.com>
  Lukas Puehringer <lukas.puehringer@nyu.edu>

<Started>
  September 25, 2018

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Provide a common interface for Python's subprocess module to:

  - namespace subprocess constants (DEVNULL, PIPE) and
  - provide a custom `subprocess.run` wrapper
  - provide a special `run_duplicate_streams` function

"""
import io
import logging
import os
import shlex
import subprocess  # nosec
import sys
import tempfile
import time

from securesystemslib import formats, settings

DEVNULL = subprocess.DEVNULL
PIPE = subprocess.PIPE

log = logging.getLogger(__name__)


def _default_timeout():
    """Helper to use securesystemslib.settings.SUBPROCESS_TIMEOUT as default
    argument, and still be able to modify it after the function definitions are
    evaluated."""
    return settings.SUBPROCESS_TIMEOUT


def run(cmd, check=True, timeout=_default_timeout(), **kwargs):
    """
    <Purpose>
      Provide wrapper for `subprocess.run` (see
      https://github.com/python/cpython/blob/3.5/Lib/subprocess.py#L352-L399)
      where:

      * `timeout` has a default (securesystemslib.settings.SUBPROCESS_TIMEOUT),
      * `check` is `True` by default,
      * there is only one positional argument, i.e. `cmd` that can be either
        a str (will be split with shlex) or a list of str and
      * instead of raising a ValueError if both `input` and `stdin` are passed,
        `stdin` is ignored.


    <Arguments>
      cmd:
              The command and its arguments. (list of str, or str)
              Splits a string specifying a command and its argument into a list
              of substrings, if necessary.

      check: (default True)
              "If check is true, and the process exits with a non-zero exit code,
              a CalledProcessError exception will be raised. Attributes of that
              exception hold the arguments, the exit code, and stdout and stderr
              if they were captured."

      timeout: (default see securesystemslib.settings.SUBPROCESS_TIMEOUT)
              "The timeout argument is passed to Popen.communicate(). If the
              timeout expires, the child process will be killed and waited for.
              The TimeoutExpired exception will be re-raised after the child
              process has terminated."

      **kwargs:
              See subprocess.run and Frequently Used Arguments to Popen
              constructor for available kwargs.
              https://docs.python.org/3.5/library/subprocess.html#subprocess.run
              https://docs.python.org/3.5/library/subprocess.html#frequently-used-arguments

    <Exceptions>
      securesystemslib.exceptions.FormatError:
              If the `cmd` is a list and does not match
              securesystemslib.formats.LIST_OF_ANY_STRING_SCHEMA.

      OSError:
              If the given command is not present or non-executable.

      subprocess.TimeoutExpired:
              If the process does not terminate after timeout seconds. Default
              is `settings.SUBPROCESS_TIMEOUT`

    <Side Effects>
      The side effects of executing the given command in this environment.

    <Returns>
      A subprocess.CompletedProcess instance.

    """
    # Make list of command passed as string for convenience
    if isinstance(cmd, str):
        cmd = shlex.split(cmd)
    else:
        formats.LIST_OF_ANY_STRING_SCHEMA.check_match(cmd)

    # NOTE: The CPython implementation would raise a ValueError here, we just
    # don't pass on `stdin` if the user passes `input` and `stdin`
    # https://github.com/python/cpython/blob/3.5/Lib/subprocess.py#L378-L381
    if kwargs.get("input") is not None and "stdin" in kwargs:
        log.debug(  # pylint: disable=logging-not-lazy
            "stdin and input arguments may not both be used. "
            "Ignoring passed stdin: " + str(kwargs["stdin"])
        )
        del kwargs["stdin"]

    return subprocess.run(cmd, check=check, timeout=timeout, **kwargs)  # nosec


def run_duplicate_streams(cmd, timeout=_default_timeout()):
    """
    <Purpose>
      Provide a function that executes a command in a subprocess and, upon
      termination, returns its exit code and the contents of what was printed to
      its standard streams.

      * Might behave unexpectedly with interactive commands.
      * Might not duplicate output in real time, if the command buffers it (see
        e.g. `print("foo")` vs. `print("foo", flush=True)` in Python 3).

    <Arguments>
      cmd:
              The command and its arguments. (list of str, or str)
              Splits a string specifying a command and its argument into a list
              of substrings, if necessary.

      timeout: (default see settings.SUBPROCESS_TIMEOUT)
              If the timeout expires, the child process will be killed and waited
              for and then subprocess.TimeoutExpired will be raised.

    <Exceptions>
      securesystemslib.exceptions.FormatError:
              If the `cmd` is a list and does not match
              securesystemslib.formats.LIST_OF_ANY_STRING_SCHEMA.

      OSError:
              If the given command is not present or non-executable.

      subprocess.TimeoutExpired:
              If the process does not terminate after timeout seconds. Default
              is `settings.SUBPROCESS_TIMEOUT`

    <Side Effects>
      The side effects of executing the given command in this environment.

    <Returns>
      A tuple of command's exit code, standard output and standard error
      contents.

    """
    if isinstance(cmd, str):
        cmd = shlex.split(cmd)
    else:
        formats.LIST_OF_ANY_STRING_SCHEMA.check_match(cmd)

    # Use temporary files as targets for child process standard stream redirects
    # They seem to work better (i.e. do not hang) than pipes, when using
    # interactive commands like `vi`.
    stdout_fd, stdout_name = tempfile.mkstemp()
    stderr_fd, stderr_name = tempfile.mkstemp()
    try:
        with io.open(  # pylint: disable=unspecified-encoding
            stdout_name, "r"
        ) as stdout_reader, os.fdopen(  # pylint: disable=unspecified-encoding
            stdout_fd, "w"
        ) as stdout_writer, io.open(  # pylint: disable=unspecified-encoding
            stderr_name, "r"
        ) as stderr_reader, os.fdopen(
            stderr_fd, "w"
        ) as stderr_writer:

            # Store stream results in mutable dict to update it inside nested helper
            _std = {"out": "", "err": ""}  # pylint: disable=invalid-name

            def _duplicate_streams():
                """Helper to read from child process standard streams, write their
                contents to parent process standard streams, and build up return values
                for outer function.
                """
                # Read until EOF but at most `io.DEFAULT_BUFFER_SIZE` bytes per call.
                # Reading and writing in reasonably sized chunks prevents us from
                # subverting a timeout, due to being busy for too long or indefinitely.
                stdout_part = stdout_reader.read(io.DEFAULT_BUFFER_SIZE)
                stderr_part = stderr_reader.read(io.DEFAULT_BUFFER_SIZE)
                sys.stdout.write(stdout_part)
                sys.stderr.write(stderr_part)
                sys.stdout.flush()
                sys.stderr.flush()
                _std["out"] += stdout_part
                _std["err"] += stderr_part

            # Start child process, writing its standard streams to temporary files
            proc = subprocess.Popen(  # pylint: disable=consider-using-with  # nosec
                cmd,
                stdout=stdout_writer,
                stderr=stderr_writer,
                universal_newlines=True,
            )
            proc_start_time = time.time()

            # Duplicate streams until the process exits (or times out)
            while proc.poll() is None:
                # Time out as Python's `subprocess` would do it
                if (
                    timeout is not None
                    and time.time() > proc_start_time + timeout
                ):
                    proc.kill()
                    proc.wait()
                    raise subprocess.TimeoutExpired(cmd, timeout)

                _duplicate_streams()

            # Read/write once more to grab everything that the process wrote between
            # our last read in the loop and exiting, i.e. breaking the loop.
            _duplicate_streams()

    finally:
        # The work is done or was interrupted, the temp files can be removed
        os.remove(stdout_name)
        os.remove(stderr_name)

    # Return process exit code and captured streams
    return proc.poll(), _std["out"], _std["err"]
